@@ -45,56 +45,56 @@ QVariant MapToVariantConverter::toVariant(const Map &map, const QDir &mapDir)
 
     QVariantMap mapVariant;
 
-    mapVariant[QLatin1String("type")] = QLatin1String("map");
-    mapVariant[QLatin1String("version")] = (mVersion == 2) ? 1.4 : 1.1;
-    mapVariant[QLatin1String("tiledversion")] = QCoreApplication::applicationVersion();
-    mapVariant[QLatin1String("orientation")] = orientationToString(map.orientation());
-    mapVariant[QLatin1String("renderorder")] = renderOrderToString(map.renderOrder());
-    mapVariant[QLatin1String("width")] = map.width();
-    mapVariant[QLatin1String("height")] = map.height();
-    mapVariant[QLatin1String("tilewidth")] = map.tileWidth();
-    mapVariant[QLatin1String("tileheight")] = map.tileHeight();
-    mapVariant[QLatin1String("infinite")] = map.infinite();
-    mapVariant[QLatin1String("nextlayerid")] = map.nextLayerId();
-    mapVariant[QLatin1String("nextobjectid")] = map.nextObjectId();
-    mapVariant[QLatin1String("compressionlevel")] = map.compressionLevel();
+    mapVariant[QStringLiteral("type")] = QLatin1String("map");
+    mapVariant[QStringLiteral("version")] = (mVersion == 2) ? 1.4 : 1.1;
+    mapVariant[QStringLiteral("tiledversion")] = QCoreApplication::applicationVersion();
+    mapVariant[QStringLiteral("orientation")] = orientationToString(map.orientation());
+    mapVariant[QStringLiteral("renderorder")] = renderOrderToString(map.renderOrder());
+    mapVariant[QStringLiteral("width")] = map.width();
+    mapVariant[QStringLiteral("height")] = map.height();
+    mapVariant[QStringLiteral("tilewidth")] = map.tileWidth();
+    mapVariant[QStringLiteral("tileheight")] = map.tileHeight();
+    mapVariant[QStringLiteral("infinite")] = map.infinite();
+    mapVariant[QStringLiteral("nextlayerid")] = map.nextLayerId();
+    mapVariant[QStringLiteral("nextobjectid")] = map.nextObjectId();
+    mapVariant[QStringLiteral("compressionlevel")] = map.compressionLevel();
 
     if (map.chunkSize() != QSize(CHUNK_SIZE, CHUNK_SIZE) || !map.exportFileName.isEmpty() || !map.exportFormat.isEmpty()) {
         QVariantMap editorSettingsVariant;
 
         if (map.chunkSize() != QSize(CHUNK_SIZE, CHUNK_SIZE)) {
             QVariantMap chunkSizeVariant;
-            chunkSizeVariant[QLatin1String("width")] = map.chunkSize().width();
-            chunkSizeVariant[QLatin1String("height")] = map.chunkSize().height();
-            editorSettingsVariant[QLatin1String("chunksize")] = chunkSizeVariant;
+            chunkSizeVariant[QStringLiteral("width")] = map.chunkSize().width();
+            chunkSizeVariant[QStringLiteral("height")] = map.chunkSize().height();
+            editorSettingsVariant[QStringLiteral("chunksize")] = chunkSizeVariant;
         }
 
         if (!map.exportFileName.isEmpty() || !map.exportFormat.isEmpty()) {
             QVariantMap exportVariant;
             if (!map.exportFileName.isEmpty())
-                exportVariant[QLatin1String("target")] = mDir.relativeFilePath(map.exportFileName);
+                exportVariant[QStringLiteral("target")] = mDir.relativeFilePath(map.exportFileName);
             if (!map.exportFormat.isEmpty())
-                exportVariant[QLatin1String("format")] = map.exportFormat;
-            editorSettingsVariant[QLatin1String("export")] = exportVariant;
+                exportVariant[QStringLiteral("format")] = map.exportFormat;
+            editorSettingsVariant[QStringLiteral("export")] = exportVariant;
         }
 
-        mapVariant[QLatin1String("editorsettings")] = editorSettingsVariant;
+        mapVariant[QStringLiteral("editorsettings")] = editorSettingsVariant;
     }
 
     addProperties(mapVariant, map.properties());
 
     if (map.orientation() == Map::Hexagonal) {
-        mapVariant[QLatin1String("hexsidelength")] = map.hexSideLength();
+        mapVariant[QStringLiteral("hexsidelength")] = map.hexSideLength();
     }
 
     if (map.orientation() == Map::Hexagonal || map.orientation() == Map::Staggered) {
-        mapVariant[QLatin1String("staggeraxis")] = staggerAxisToString(map.staggerAxis());
-        mapVariant[QLatin1String("staggerindex")] = staggerIndexToString(map.staggerIndex());
+        mapVariant[QStringLiteral("staggeraxis")] = staggerAxisToString(map.staggerAxis());
+        mapVariant[QStringLiteral("staggerindex")] = staggerIndexToString(map.staggerIndex());
     }
 
     const QColor bgColor = map.backgroundColor();
     if (bgColor.isValid())
-        mapVariant[QLatin1String("backgroundcolor")] = colorToString(bgColor);
+        mapVariant[QStringLiteral("backgroundcolor")] = colorToString(bgColor);
 
     QVariantList tilesetVariants;
 
@@ -104,9 +104,9 @@ QVariant MapToVariantConverter::toVariant(const Map &map, const QDir &mapDir)
         mGidMapper.insert(firstGid, tileset);
         firstGid += tileset->nextTileId();
     }
-    mapVariant[QLatin1String("tilesets")] = tilesetVariants;
+    mapVariant[QStringLiteral("tilesets")] = tilesetVariants;
 
-    mapVariant[QLatin1String("layers")] = toVariant(map.layers(),
+    mapVariant[QStringLiteral("layers")] = toVariant(map.layers(),
                                                     map.layerDataFormat(),
                                                     map.compressionLevel(),
                                                     map.chunkSize());
@@ -127,16 +127,16 @@ QVariant MapToVariantConverter::toVariant(const ObjectTemplate &objectTemplate,
     mDir = directory;
     QVariantMap objectTemplateVariant;
 
-    objectTemplateVariant[QLatin1String("type")] = QLatin1String("template");
+    objectTemplateVariant[QStringLiteral("type")] = QLatin1String("template");
 
     mGidMapper.clear();
     if (Tileset *tileset = objectTemplate.object()->cell().tileset()) {
         unsigned firstGid = 1;
         mGidMapper.insert(firstGid, tileset->sharedPointer());
-        objectTemplateVariant[QLatin1String("tileset")] = toVariant(*tileset, firstGid);
+        objectTemplateVariant[QStringLiteral("tileset")] = toVariant(*tileset, firstGid);
     }
 
-    objectTemplateVariant[QLatin1String("object")] = toVariant(*objectTemplate.object());
+    objectTemplateVariant[QStringLiteral("object")] = toVariant(*objectTemplate.object());
 
     return objectTemplateVariant;
 }
@@ -147,32 +147,32 @@ QVariant MapToVariantConverter::toVariant(const Tileset &tileset,
     QVariantMap tilesetVariant;
 
     if (firstGid > 0) {
-        tilesetVariant[QLatin1String("firstgid")] = firstGid;
+        tilesetVariant[QStringLiteral("firstgid")] = firstGid;
 
         const QString &fileName = tileset.fileName();
         if (!fileName.isEmpty()) {
             QString source = mDir.relativeFilePath(fileName);
-            tilesetVariant[QLatin1String("source")] = source;
+            tilesetVariant[QStringLiteral("source")] = source;
 
             // Tileset is external, so no need to write any of the stuff below
             return tilesetVariant;
         }
     } else {
         // Include a 'type' property if we are writing the tileset to its own file
-        tilesetVariant[QLatin1String("type")] = QLatin1String("tileset");
+        tilesetVariant[QStringLiteral("type")] = QLatin1String("tileset");
 
         // Include version in external tilesets
-        tilesetVariant[QLatin1String("version")] = (mVersion == 2) ? 1.4 : 1.1;
-        tilesetVariant[QLatin1String("tiledversion")] = QCoreApplication::applicationVersion();
+        tilesetVariant[QStringLiteral("version")] = (mVersion == 2) ? 1.4 : 1.1;
+        tilesetVariant[QStringLiteral("tiledversion")] = QCoreApplication::applicationVersion();
     }
 
-    tilesetVariant[QLatin1String("name")] = tileset.name();
-    tilesetVariant[QLatin1String("tilewidth")] = tileset.tileWidth();
-    tilesetVariant[QLatin1String("tileheight")] = tileset.tileHeight();
-    tilesetVariant[QLatin1String("spacing")] = tileset.tileSpacing();
-    tilesetVariant[QLatin1String("margin")] = tileset.margin();
-    tilesetVariant[QLatin1String("tilecount")] = tileset.tileCount();
-    tilesetVariant[QLatin1String("columns")] = tileset.columnCount();
+    tilesetVariant[QStringLiteral("name")] = tileset.name();
+    tilesetVariant[QStringLiteral("tilewidth")] = tileset.tileWidth();
+    tilesetVariant[QStringLiteral("tileheight")] = tileset.tileHeight();
+    tilesetVariant[QStringLiteral("spacing")] = tileset.tileSpacing();
+    tilesetVariant[QStringLiteral("margin")] = tileset.margin();
+    tilesetVariant[QStringLiteral("tilecount")] = tileset.tileCount();
+    tilesetVariant[QStringLiteral("columns")] = tileset.columnCount();
 
     // Write editor settings when saving external tilesets
     if (firstGid == 0) {
@@ -180,37 +180,37 @@ QVariant MapToVariantConverter::toVariant(const Tileset &tileset,
             QVariantMap editorSettingsVariant;
 
             QVariantMap exportVariant;
-            exportVariant[QLatin1String("target")] = mDir.relativeFilePath(tileset.exportFileName);
-            exportVariant[QLatin1String("format")] = tileset.exportFormat;
-            editorSettingsVariant[QLatin1String("export")] = exportVariant;
+            exportVariant[QStringLiteral("target")] = mDir.relativeFilePath(tileset.exportFileName);
+            exportVariant[QStringLiteral("format")] = tileset.exportFormat;
+            editorSettingsVariant[QStringLiteral("export")] = exportVariant;
 
-            tilesetVariant[QLatin1String("editorsettings")] = editorSettingsVariant;
+            tilesetVariant[QStringLiteral("editorsettings")] = editorSettingsVariant;
         }
     }
 
     const QColor &backgroundColor = tileset.backgroundColor();
     if (backgroundColor.isValid())
-        tilesetVariant[QLatin1String("backgroundcolor")] = colorToString(backgroundColor);
+        tilesetVariant[QStringLiteral("backgroundcolor")] = colorToString(backgroundColor);
 
     if (tileset.objectAlignment() != Unspecified)
-        tilesetVariant[QLatin1String("objectalignment")] = alignmentToString(tileset.objectAlignment());
+        tilesetVariant[QStringLiteral("objectalignment")] = alignmentToString(tileset.objectAlignment());
 
     addProperties(tilesetVariant, tileset.properties());
 
     const QPoint offset = tileset.tileOffset();
     if (!offset.isNull()) {
         QVariantMap tileOffset;
-        tileOffset[QLatin1String("x")] = offset.x();
-        tileOffset[QLatin1String("y")] = offset.y();
-        tilesetVariant[QLatin1String("tileoffset")] = tileOffset;
+        tileOffset[QStringLiteral("x")] = offset.x();
+        tileOffset[QStringLiteral("y")] = offset.y();
+        tilesetVariant[QStringLiteral("tileoffset")] = tileOffset;
     }
 
     if (tileset.orientation() != Tileset::Orthogonal || tileset.gridSize() != tileset.tileSize()) {
         QVariantMap grid;
-        grid[QLatin1String("orientation")] = Tileset::orientationToString(tileset.orientation());
-        grid[QLatin1String("width")] = tileset.gridSize().width();
-        grid[QLatin1String("height")] = tileset.gridSize().height();
-        tilesetVariant[QLatin1String("grid")] = grid;
+        grid[QStringLiteral("orientation")] = Tileset::orientationToString(tileset.orientation());
+        grid[QStringLiteral("width")] = tileset.gridSize().width();
+        grid[QStringLiteral("height")] = tileset.gridSize().height();
+        tilesetVariant[QStringLiteral("grid")] = grid;
     }
 
     // Write the image element
@@ -218,14 +218,14 @@ QVariant MapToVariantConverter::toVariant(const Tileset &tileset,
     if (!imageSource.isEmpty()) {
         const QString rel = toFileReference(imageSource, mDir);
 
-        tilesetVariant[QLatin1String("image")] = rel;
+        tilesetVariant[QStringLiteral("image")] = rel;
 
         const QColor transColor = tileset.transparentColor();
         if (transColor.isValid())
-            tilesetVariant[QLatin1String("transparentcolor")] = transColor.name();
+            tilesetVariant[QStringLiteral("transparentcolor")] = transColor.name();
 
-        tilesetVariant[QLatin1String("imagewidth")] = tileset.imageWidth();
-        tilesetVariant[QLatin1String("imageheight")] = tileset.imageHeight();
+        tilesetVariant[QStringLiteral("imagewidth")] = tileset.imageWidth();
+        tilesetVariant[QStringLiteral("imageheight")] = tileset.imageHeight();
     }
 
     // Write the properties, terrain, external image, object group and
@@ -253,57 +253,57 @@ QVariant MapToVariantConverter::toVariant(const Tileset &tileset,
         }
 
         if (!tile->type().isEmpty())
-            tileVariant[QLatin1String("type")] = tile->type();
+            tileVariant[QStringLiteral("type")] = tile->type();
         if (tile->terrain() != 0xFFFFFFFF) {
             QVariantList terrainIds;
             for (int j = 0; j < 4; ++j)
                 terrainIds << QVariant(tile->cornerTerrainId(j));
-            tileVariant[QLatin1String("terrain")] = terrainIds;
+            tileVariant[QStringLiteral("terrain")] = terrainIds;
         }
         if (tile->probability() != 1.0)
-            tileVariant[QLatin1String("probability")] = tile->probability();
+            tileVariant[QStringLiteral("probability")] = tile->probability();
         if (!tile->imageSource().isEmpty()) {
             const QString rel = toFileReference(tile->imageSource(), mDir);
-            tileVariant[QLatin1String("image")] = rel;
+            tileVariant[QStringLiteral("image")] = rel;
 
             const QSize tileSize = tile->size();
             if (!tileSize.isNull()) {
-                tileVariant[QLatin1String("imagewidth")] = tileSize.width();
-                tileVariant[QLatin1String("imageheight")] = tileSize.height();
+                tileVariant[QStringLiteral("imagewidth")] = tileSize.width();
+                tileVariant[QStringLiteral("imageheight")] = tileSize.height();
             }
         }
         if (tile->objectGroup())
-            tileVariant[QLatin1String("objectgroup")] = toVariant(*tile->objectGroup());
+            tileVariant[QStringLiteral("objectgroup")] = toVariant(*tile->objectGroup());
         if (tile->isAnimated()) {
             QVariantList frameVariants;
             for (const Frame &frame : tile->frames()) {
                 QVariantMap frameVariant;
-                frameVariant[QLatin1String("tileid")] = frame.tileId;
-                frameVariant[QLatin1String("duration")] = frame.duration;
+                frameVariant[QStringLiteral("tileid")] = frame.tileId;
+                frameVariant[QStringLiteral("duration")] = frame.duration;
                 frameVariants.append(frameVariant);
             }
-            tileVariant[QLatin1String("animation")] = frameVariants;
+            tileVariant[QStringLiteral("animation")] = frameVariants;
         }
 
         if (!tileVariant.empty()) {
             if (mVersion == 1) {
                 tilesVariantMap[QString::number(tile->id())] = tileVariant;
             } else {
-                tileVariant[QLatin1String("id")] = tile->id();
+                tileVariant[QStringLiteral("id")] = tile->id();
                 tilesVariant << tileVariant;
             }
         }
     }
 
     if (!tilePropertiesVariant.empty()) {
-        tilesetVariant[QLatin1String("tileproperties")] = tilePropertiesVariant;
-        tilesetVariant[QLatin1String("tilepropertytypes")] = tilePropertyTypesVariant;
+        tilesetVariant[QStringLiteral("tileproperties")] = tilePropertiesVariant;
+        tilesetVariant[QStringLiteral("tilepropertytypes")] = tilePropertyTypesVariant;
     }
 
     if (!tilesVariantMap.empty())
-        tilesetVariant[QLatin1String("tiles")] = tilesVariantMap;
+        tilesetVariant[QStringLiteral("tiles")] = tilesVariantMap;
     else if (!tilesVariant.empty())
-        tilesetVariant[QLatin1String("tiles")] = tilesVariant;
+        tilesetVariant[QStringLiteral("tiles")] = tilesVariant;
 
     // Write terrains
     if (tileset.terrainCount() > 0) {
@@ -312,12 +312,12 @@ QVariant MapToVariantConverter::toVariant(const Tileset &tileset,
             Terrain *terrain = tileset.terrain(i);
             const Properties &properties = terrain->properties();
             QVariantMap terrainVariant;
-            terrainVariant[QLatin1String("name")] = terrain->name();
-            terrainVariant[QLatin1String("tile")] = terrain->imageTileId();
+            terrainVariant[QStringLiteral("name")] = terrain->name();
+            terrainVariant[QStringLiteral("tile")] = terrain->imageTileId();
             addProperties(terrainVariant, properties);
             terrainsVariant << terrainVariant;
         }
-        tilesetVariant[QLatin1String("terrains")] = terrainsVariant;
+        tilesetVariant[QStringLiteral("terrains")] = terrainsVariant;
     }
 
     // Write the Wang sets
@@ -327,7 +327,7 @@ QVariant MapToVariantConverter::toVariant(const Tileset &tileset,
         for (const WangSet *wangSet : tileset.wangSets())
             wangSetVariants.append(toVariant(*wangSet));
 
-        tilesetVariant[QLatin1String("wangsets")] = wangSetVariants;
+        tilesetVariant[QStringLiteral("wangsets")] = wangSetVariants;
     }
 
     return tilesetVariant;
@@ -364,8 +364,8 @@ QVariant MapToVariantConverter::toVariant(const WangSet &wangSet) const
 {
     QVariantMap wangSetVariant;
 
-    wangSetVariant[QLatin1String("name")] = wangSet.name();
-    wangSetVariant[QLatin1String("tile")] = wangSet.imageTileId();
+    wangSetVariant[QStringLiteral("name")] = wangSet.name();
+    wangSetVariant[QStringLiteral("tile")] = wangSet.imageTileId();
 
     QVariantList edgeColorVariants;
     if (wangSet.edgeColorCount() > 1) {
@@ -374,7 +374,7 @@ QVariant MapToVariantConverter::toVariant(const WangSet &wangSet) const
                 edgeColorVariants.append(toVariant(*wc));
         }
     }
-    wangSetVariant[QLatin1String("edgecolors")] = edgeColorVariants;
+    wangSetVariant[QStringLiteral("edgecolors")] = edgeColorVariants;
 
     QVariantList cornerColorVariants;
     if (wangSet.cornerColorCount() > 1) {
@@ -383,7 +383,7 @@ QVariant MapToVariantConverter::toVariant(const WangSet &wangSet) const
                 cornerColorVariants.append(toVariant(*wc));
         }
     }
-    wangSetVariant[QLatin1String("cornercolors")] = cornerColorVariants;
+    wangSetVariant[QStringLiteral("cornercolors")] = cornerColorVariants;
 
     QVariantList wangTileVariants;
     const auto wangTiles = wangSet.sortedWangTiles();
@@ -394,15 +394,15 @@ QVariant MapToVariantConverter::toVariant(const WangSet &wangSet) const
         for (int i = 0; i < 8; ++i)
             wangIdVariant.append(QVariant(wangTile.wangId().indexColor(i)));
 
-        wangTileVariant[QLatin1String("wangid")] = wangIdVariant;
-        wangTileVariant[QLatin1String("tileid")] = wangTile.tile()->id();
-        wangTileVariant[QLatin1String("hflip")] = wangTile.flippedHorizontally();
-        wangTileVariant[QLatin1String("vflip")] = wangTile.flippedVertically();
-        wangTileVariant[QLatin1String("dflip")] = wangTile.flippedAntiDiagonally();
+        wangTileVariant[QStringLiteral("wangid")] = wangIdVariant;
+        wangTileVariant[QStringLiteral("tileid")] = wangTile.tile()->id();
+        wangTileVariant[QStringLiteral("hflip")] = wangTile.flippedHorizontally();
+        wangTileVariant[QStringLiteral("vflip")] = wangTile.flippedVertically();
+        wangTileVariant[QStringLiteral("dflip")] = wangTile.flippedAntiDiagonally();
 
         wangTileVariants.append(wangTileVariant);
     }
-    wangSetVariant[QLatin1String("wangtiles")] = wangTileVariants;
+    wangSetVariant[QStringLiteral("wangtiles")] = wangTileVariants;
 
     addProperties(wangSetVariant, wangSet.properties());
 
@@ -412,10 +412,10 @@ QVariant MapToVariantConverter::toVariant(const WangSet &wangSet) const
 QVariant MapToVariantConverter::toVariant(const WangColor &wangColor) const
 {
     QVariantMap colorVariant;
-    colorVariant[QLatin1String("color")] = colorToString(wangColor.color());
-    colorVariant[QLatin1String("name")] = wangColor.name();
-    colorVariant[QLatin1String("probability")] = wangColor.probability();
-    colorVariant[QLatin1String("tile")] = wangColor.imageId();
+    colorVariant[QStringLiteral("color")] = colorToString(wangColor.color());
+    colorVariant[QStringLiteral("name")] = wangColor.name();
+    colorVariant[QStringLiteral("probability")] = wangColor.probability();
+    colorVariant[QStringLiteral("tile")] = wangColor.imageId();
     return colorVariant;
 }
 
@@ -451,17 +451,17 @@ QVariant MapToVariantConverter::toVariant(const TileLayer &tileLayer,
                                           QSize chunkSize) const
 {
     QVariantMap tileLayerVariant;
-    tileLayerVariant[QLatin1String("type")] = QLatin1String("tilelayer");
+    tileLayerVariant[QStringLiteral("type")] = QLatin1String("tilelayer");
 
     if (tileLayer.map()->infinite()) {
         QRect bounds = tileLayer.localBounds();
-        tileLayerVariant[QLatin1String("width")] = bounds.width();
-        tileLayerVariant[QLatin1String("height")] = bounds.height();
-        tileLayerVariant[QLatin1String("startx")] = bounds.left();
-        tileLayerVariant[QLatin1String("starty")] = bounds.top();
+        tileLayerVariant[QStringLiteral("width")] = bounds.width();
+        tileLayerVariant[QStringLiteral("height")] = bounds.height();
+        tileLayerVariant[QStringLiteral("startx")] = bounds.left();
+        tileLayerVariant[QStringLiteral("starty")] = bounds.top();
     } else {
-        tileLayerVariant[QLatin1String("width")] = tileLayer.width();
-        tileLayerVariant[QLatin1String("height")] = tileLayer.height();
+        tileLayerVariant[QStringLiteral("width")] = tileLayer.width();
+        tileLayerVariant[QStringLiteral("height")] = tileLayer.height();
     }
 
     addLayerAttributes(tileLayerVariant, tileLayer);
@@ -474,8 +474,8 @@ QVariant MapToVariantConverter::toVariant(const TileLayer &tileLayer,
     case Map::Base64Zlib:
     case Map::Base64Gzip:
     case Map::Base64Zstandard:
-        tileLayerVariant[QLatin1String("encoding")] = QLatin1String("base64");
-        tileLayerVariant[QLatin1String("compression")] = compressionToString(format);
+        tileLayerVariant[QStringLiteral("encoding")] = QLatin1String("base64");
+        tileLayerVariant[QStringLiteral("compression")] = compressionToString(format);
         break;
     }
 
@@ -486,17 +486,17 @@ QVariant MapToVariantConverter::toVariant(const TileLayer &tileLayer,
         for (const QRect &rect : chunks) {
             QVariantMap chunkVariant;
 
-            chunkVariant[QLatin1String("x")] = rect.x();
-            chunkVariant[QLatin1String("y")] = rect.y();
-            chunkVariant[QLatin1String("width")] = rect.width();
-            chunkVariant[QLatin1String("height")] = rect.height();
+            chunkVariant[QStringLiteral("x")] = rect.x();
+            chunkVariant[QStringLiteral("y")] = rect.y();
+            chunkVariant[QStringLiteral("width")] = rect.width();
+            chunkVariant[QStringLiteral("height")] = rect.height();
 
             addTileLayerData(chunkVariant, tileLayer, format, compressionLevel, rect);
 
             chunkVariants.append(chunkVariant);
         }
 
-        tileLayerVariant[QLatin1String("chunks")] = chunkVariants;
+        tileLayerVariant[QStringLiteral("chunks")] = chunkVariants;
     } else {
         addTileLayerData(tileLayerVariant, tileLayer, format, compressionLevel,
                          QRect(0, 0, tileLayer.width(), tileLayer.height()));
@@ -508,19 +508,19 @@ QVariant MapToVariantConverter::toVariant(const TileLayer &tileLayer,
 QVariant MapToVariantConverter::toVariant(const ObjectGroup &objectGroup) const
 {
     QVariantMap objectGroupVariant;
-    objectGroupVariant[QLatin1String("type")] = QLatin1String("objectgroup");
+    objectGroupVariant[QStringLiteral("type")] = QLatin1String("objectgroup");
 
     if (objectGroup.color().isValid())
-        objectGroupVariant[QLatin1String("color")] = colorToString(objectGroup.color());
+        objectGroupVariant[QStringLiteral("color")] = colorToString(objectGroup.color());
 
-    objectGroupVariant[QLatin1String("draworder")] = drawOrderToString(objectGroup.drawOrder());
+    objectGroupVariant[QStringLiteral("draworder")] = drawOrderToString(objectGroup.drawOrder());
 
     addLayerAttributes(objectGroupVariant, objectGroup);
     QVariantList objectVariants;
     for (const MapObject *object : objectGroup.objects())
         objectVariants << toVariant(*object);
 
-    objectGroupVariant[QLatin1String("objects")] = objectVariants;
+    objectGroupVariant[QStringLiteral("objects")] = objectVariants;
 
     return objectGroupVariant;
 }
@@ -535,41 +535,41 @@ QVariant MapToVariantConverter::toVariant(const MapObject &object) const
 
     if (const ObjectTemplate *objectTemplate = object.objectTemplate()) {
         QString relativeFileName = mDir.relativeFilePath(objectTemplate->fileName());
-        objectVariant[QLatin1String("template")] = relativeFileName;
+        objectVariant[QStringLiteral("template")] = relativeFileName;
     }
 
     bool notTemplateInstance = !object.isTemplateInstance();
 
     int id = object.id();
     if (id != 0)
-        objectVariant[QLatin1String("id")] = id;
+        objectVariant[QStringLiteral("id")] = id;
 
     if (notTemplateInstance || object.propertyChanged(MapObject::NameProperty))
-        objectVariant[QLatin1String("name")] = name;
+        objectVariant[QStringLiteral("name")] = name;
 
     if (notTemplateInstance || object.propertyChanged(MapObject::TypeProperty))
-        objectVariant[QLatin1String("type")] = type;
+        objectVariant[QStringLiteral("type")] = type;
 
 
     if (notTemplateInstance || object.propertyChanged(MapObject::CellProperty))
         if (!object.cell().isEmpty())
-            objectVariant[QLatin1String("gid")] = mGidMapper.cellToGid(object.cell());
+            objectVariant[QStringLiteral("gid")] = mGidMapper.cellToGid(object.cell());
 
     if (!object.isTemplateBase()) {
-        objectVariant[QLatin1String("x")] = object.x();
-        objectVariant[QLatin1String("y")] = object.y();
+        objectVariant[QStringLiteral("x")] = object.x();
+        objectVariant[QStringLiteral("y")] = object.y();
     }
 
     if (notTemplateInstance || object.propertyChanged(MapObject::SizeProperty)) {
-        objectVariant[QLatin1String("width")] = object.width();
-        objectVariant[QLatin1String("height")] = object.height();
+        objectVariant[QStringLiteral("width")] = object.width();
+        objectVariant[QStringLiteral("height")] = object.height();
     }
 
     if (notTemplateInstance || object.propertyChanged(MapObject::RotationProperty))
-        objectVariant[QLatin1String("rotation")] = object.rotation();
+        objectVariant[QStringLiteral("rotation")] = object.rotation();
 
     if (notTemplateInstance || object.propertyChanged(MapObject::VisibleProperty))
-        objectVariant[QLatin1String("visible")] = object.isVisible();
+        objectVariant[QStringLiteral("visible")] = object.isVisible();
 
     /* Polygons are stored in this format:
      *
@@ -588,21 +588,21 @@ QVariant MapToVariantConverter::toVariant(const MapObject &object) const
             QVariantList pointVariants;
             for (const QPointF &point : object.polygon()) {
                 QVariantMap pointVariant;
-                pointVariant[QLatin1String("x")] = point.x();
-                pointVariant[QLatin1String("y")] = point.y();
+                pointVariant[QStringLiteral("x")] = point.x();
+                pointVariant[QStringLiteral("y")] = point.y();
                 pointVariants.append(pointVariant);
             }
 
             if (object.shape() == MapObject::Polygon)
-                objectVariant[QLatin1String("polygon")] = pointVariants;
+                objectVariant[QStringLiteral("polygon")] = pointVariants;
             else
-                objectVariant[QLatin1String("polyline")] = pointVariants;
+                objectVariant[QStringLiteral("polyline")] = pointVariants;
         }
         break;
     }
     case MapObject::Ellipse:
         if (notTemplateInstance || object.propertyChanged(MapObject::ShapeProperty))
-            objectVariant[QLatin1String("ellipse")] = true;
+            objectVariant[QStringLiteral("ellipse")] = true;
         break;
     case MapObject::Text:
         if (notTemplateInstance || (object.propertyChanged(MapObject::TextProperty) ||
@@ -610,11 +610,11 @@ QVariant MapToVariantConverter::toVariant(const MapObject &object) const
                                     object.propertyChanged(MapObject::TextAlignmentProperty) ||
                                     object.propertyChanged(MapObject::TextWordWrapProperty) ||
                                     object.propertyChanged(MapObject::TextColorProperty)))
-            objectVariant[QLatin1String("text")] = toVariant(object.textData());
+            objectVariant[QStringLiteral("text")] = toVariant(object.textData());
         break;
     case MapObject::Point:
         if (notTemplateInstance || object.propertyChanged(MapObject::ShapeProperty))
-            objectVariant[QLatin1String("point")] = true;
+            objectVariant[QStringLiteral("point")] = true;
         break;
     }
 
@@ -625,41 +625,41 @@ QVariant MapToVariantConverter::toVariant(const TextData &textData) const
 {
     QVariantMap textVariant;
 
-    textVariant[QLatin1String("text")] = textData.text;
+    textVariant[QStringLiteral("text")] = textData.text;
 
     if (textData.font.family() != QLatin1String("sans-serif"))
-        textVariant[QLatin1String("fontfamily")] = textData.font.family();
+        textVariant[QStringLiteral("fontfamily")] = textData.font.family();
     if (textData.font.pixelSize() >= 0 && textData.font.pixelSize() != 16)
-        textVariant[QLatin1String("pixelsize")] = textData.font.pixelSize();
+        textVariant[QStringLiteral("pixelsize")] = textData.font.pixelSize();
     if (textData.wordWrap)
-        textVariant[QLatin1String("wrap")] = textData.wordWrap;
+        textVariant[QStringLiteral("wrap")] = textData.wordWrap;
     if (textData.color != Qt::black)
-        textVariant[QLatin1String("color")] = colorToString(textData.color);
+        textVariant[QStringLiteral("color")] = colorToString(textData.color);
     if (textData.font.bold())
-        textVariant[QLatin1String("bold")] = textData.font.bold();
+        textVariant[QStringLiteral("bold")] = textData.font.bold();
     if (textData.font.italic())
-        textVariant[QLatin1String("italic")] = textData.font.italic();
+        textVariant[QStringLiteral("italic")] = textData.font.italic();
     if (textData.font.underline())
-        textVariant[QLatin1String("underline")] = textData.font.underline();
+        textVariant[QStringLiteral("underline")] = textData.font.underline();
     if (textData.font.strikeOut())
-        textVariant[QLatin1String("strikeout")] = textData.font.strikeOut();
+        textVariant[QStringLiteral("strikeout")] = textData.font.strikeOut();
     if (!textData.font.kerning())
-        textVariant[QLatin1String("kerning")] = textData.font.kerning();
+        textVariant[QStringLiteral("kerning")] = textData.font.kerning();
 
     if (!textData.alignment.testFlag(Qt::AlignLeft)) {
         if (textData.alignment.testFlag(Qt::AlignHCenter))
-            textVariant[QLatin1String("halign")] = QLatin1String("center");
+            textVariant[QStringLiteral("halign")] = QLatin1String("center");
         else if (textData.alignment.testFlag(Qt::AlignRight))
-            textVariant[QLatin1String("halign")] = QLatin1String("right");
+            textVariant[QStringLiteral("halign")] = QLatin1String("right");
         else if (textData.alignment.testFlag(Qt::AlignJustify))
-            textVariant[QLatin1String("halign")] = QLatin1String("justify");
+            textVariant[QStringLiteral("halign")] = QLatin1String("justify");
     }
 
     if (!textData.alignment.testFlag(Qt::AlignTop)) {
         if (textData.alignment.testFlag(Qt::AlignVCenter))
-            textVariant[QLatin1String("valign")] = QLatin1String("center");
+            textVariant[QStringLiteral("valign")] = QLatin1String("center");
         else if (textData.alignment.testFlag(Qt::AlignBottom))
-            textVariant[QLatin1String("valign")] = QLatin1String("bottom");
+            textVariant[QStringLiteral("valign")] = QLatin1String("bottom");
     }
 
     return textVariant;
@@ -668,16 +668,16 @@ QVariant MapToVariantConverter::toVariant(const TextData &textData) const
 QVariant MapToVariantConverter::toVariant(const ImageLayer &imageLayer) const
 {
     QVariantMap imageLayerVariant;
-    imageLayerVariant[QLatin1String("type")] = QLatin1String("imagelayer");
+    imageLayerVariant[QStringLiteral("type")] = QLatin1String("imagelayer");
 
     addLayerAttributes(imageLayerVariant, imageLayer);
 
     const QString rel = toFileReference(imageLayer.imageSource(), mDir);
-    imageLayerVariant[QLatin1String("image")] = rel;
+    imageLayerVariant[QStringLiteral("image")] = rel;
 
     const QColor transColor = imageLayer.transparentColor();
     if (transColor.isValid())
-        imageLayerVariant[QLatin1String("transparentcolor")] = transColor.name();
+        imageLayerVariant[QStringLiteral("transparentcolor")] = transColor.name();
 
     return imageLayerVariant;
 }
@@ -688,11 +688,11 @@ QVariant MapToVariantConverter::toVariant(const GroupLayer &groupLayer,
                                           QSize chunkSize) const
 {
     QVariantMap groupLayerVariant;
-    groupLayerVariant[QLatin1String("type")] = QLatin1String("group");
+    groupLayerVariant[QStringLiteral("type")] = QLatin1String("group");
 
     addLayerAttributes(groupLayerVariant, groupLayer);
 
-    groupLayerVariant[QLatin1String("layers")] = toVariant(groupLayer.layers(),
+    groupLayerVariant[QStringLiteral("layers")] = toVariant(groupLayer.layers(),
                                                            format,
                                                            compressionLevel,
                                                            chunkSize);
@@ -714,7 +714,7 @@ void MapToVariantConverter::addTileLayerData(QVariantMap &variant,
             for (int x = bounds.left(); x <= bounds.right(); ++x)
                 tileVariants << mGidMapper.cellToGid(tileLayer.cellAt(x, y));
 
-        variant[QLatin1String("data")] = tileVariants;
+        variant[QStringLiteral("data")] = tileVariants;
         break;
     }
     case Map::Base64:
@@ -722,7 +722,7 @@ void MapToVariantConverter::addTileLayerData(QVariantMap &variant,
     case Map::Base64Gzip:
     case Map::Base64Zstandard:{
         QByteArray layerData = mGidMapper.encodeLayerData(tileLayer, format, bounds, compressionLevel);
-        variant[QLatin1String("data")] = layerData;
+        variant[QStringLiteral("data")] = layerData;
         break;
     }
     }
@@ -732,22 +732,22 @@ void MapToVariantConverter::addLayerAttributes(QVariantMap &layerVariant,
                                                const Layer &layer) const
 {
     if (layer.id() != 0)
-        layerVariant[QLatin1String("id")] = layer.id();
+        layerVariant[QStringLiteral("id")] = layer.id();
 
-    layerVariant[QLatin1String("name")] = layer.name();
-    layerVariant[QLatin1String("x")] = layer.x();
-    layerVariant[QLatin1String("y")] = layer.y();
-    layerVariant[QLatin1String("visible")] = layer.isVisible();
-    layerVariant[QLatin1String("opacity")] = layer.opacity();
+    layerVariant[QStringLiteral("name")] = layer.name();
+    layerVariant[QStringLiteral("x")] = layer.x();
+    layerVariant[QStringLiteral("y")] = layer.y();
+    layerVariant[QStringLiteral("visible")] = layer.isVisible();
+    layerVariant[QStringLiteral("opacity")] = layer.opacity();
 
     const QPointF offset = layer.offset();
     if (!offset.isNull()) {
-        layerVariant[QLatin1String("offsetx")] = offset.x();
-        layerVariant[QLatin1String("offsety")] = offset.y();
+        layerVariant[QStringLiteral("offsetx")] = offset.x();
+        layerVariant[QStringLiteral("offsety")] = offset.y();
     }
 
     if (layer.tintColor().isValid())
-        layerVariant[QLatin1String("tintcolor")] = colorToString(layer.tintColor());
+        layerVariant[QStringLiteral("tintcolor")] = colorToString(layer.tintColor());
 
     addProperties(layerVariant, layer.properties());
 }
@@ -772,8 +772,8 @@ void MapToVariantConverter::addProperties(QVariantMap &variantMap,
             propertyTypesMap[it.key()] = typeToName(type);
         }
 
-        variantMap[QLatin1String("properties")] = propertiesMap;
-        variantMap[QLatin1String("propertytypes")] = propertyTypesMap;
+        variantMap[QStringLiteral("properties")] = propertiesMap;
+        variantMap[QStringLiteral("propertytypes")] = propertyTypesMap;
     } else {
         QVariantList propertiesVariantList;
 
@@ -784,12 +784,12 @@ void MapToVariantConverter::addProperties(QVariantMap &variantMap,
             const QVariant value = toExportValue(it.value(), mDir);
 
             QVariantMap propertyVariantMap;
-            propertyVariantMap[QLatin1String("name")] = it.key();
-            propertyVariantMap[QLatin1String("value")] = value;
-            propertyVariantMap[QLatin1String("type")] = typeToName(type);
+            propertyVariantMap[QStringLiteral("name")] = it.key();
+            propertyVariantMap[QStringLiteral("value")] = value;
+            propertyVariantMap[QStringLiteral("type")] = typeToName(type);
             propertiesVariantList << propertyVariantMap;
         }
 
-        variantMap[QLatin1String("properties")] = propertiesVariantList;
+        variantMap[QStringLiteral("properties")] = propertiesVariantList;
     }
 }
