@@ -32,6 +32,7 @@
 #include "mapobject.h"
 #include "tileset.h"
 
+#include <QDateTime>
 #include <QPointer>
 
 #include <memory>
@@ -42,8 +43,6 @@ class ObjectTemplateFormat;
 
 class TILEDSHARED_EXPORT ObjectTemplate : public Object
 {
-    Q_OBJECT
-
 public:
     ObjectTemplate();
     ObjectTemplate(const QString &fileName);
@@ -51,7 +50,7 @@ public:
 
     const MapObject *object() const;
     void setObject(const MapObject *object);
-    void setObject(std::unique_ptr<MapObject> &&object);
+    void setObject(std::unique_ptr<MapObject> object);
 
     const QString &fileName() const;
     void setFileName(const QString &fileName);
@@ -59,6 +58,9 @@ public:
     void setFormat(ObjectTemplateFormat *format);
     ObjectTemplateFormat *format() const;
 
+    bool save();
+    QDateTime lastSaved() const;
+
     const SharedTileset &tileset() const;
 
 private:
@@ -66,6 +68,7 @@ private:
     QPointer<ObjectTemplateFormat> mFormat;
     std::unique_ptr<MapObject> mObject;
     SharedTileset mTileset;
+    QDateTime mLastSaved;
 };
 
 inline const MapObject *ObjectTemplate::object() const
@@ -77,6 +80,9 @@ inline const QString &ObjectTemplate::fileName() const
 inline void ObjectTemplate::setFileName(const QString &fileName)
 { mFileName = fileName; }
 
+inline QDateTime ObjectTemplate::lastSaved() const
+{ return mLastSaved; }
+
 inline const SharedTileset &ObjectTemplate::tileset() const
 { return mTileset; }
 
