@@ -32,9 +32,21 @@
 
 #include <QColor>
 #include <QJsonObject>
+#include <QVector>
 
 namespace Tiled {
 
+QString FilePath::toString(const FilePath &path)
+{
+    return path.url.toString(QUrl::PreferLocalFile);
+}
+
+FilePath FilePath::fromString(const QString &string)
+{
+    return { Tiled::toUrl(string) };
+}
+
+
 void mergeProperties(Properties &target, const Properties &source)
 {
     // Based on QMap::unite, but using insert instead of insertMulti
@@ -109,6 +121,11 @@ int filePathTypeId()
     return qMetaTypeId<FilePath>();
 }
 
+int objectRefTypeId()
+{
+    return qMetaTypeId<ObjectRef>();
+}
+
 QString typeToName(int type)
 {
     switch (type) {
@@ -121,6 +138,8 @@ QString typeToName(int type)
     default:
         if (type == filePathTypeId())
             return QStringLiteral("file");
+        if (type == objectRefTypeId())
+            return QStringLiteral("object");
     }
     return QLatin1String(QVariant::typeToName(type));
 }
@@ -135,30 +154,27 @@ int nameToType(const QString &name)
         return QVariant::Color;
     if (name == QLatin1String("file"))
         return filePathTypeId();
+    if (name == QLatin1String("object"))
+        return objectRefTypeId();
 
     return QVariant::nameToType(name.toLatin1().constData());
 }
 
-static QString colorToString(const QColor &color)
-{
-    if (!color.isValid())
-        return QString();
-
-    return color.name(QColor::HexArgb);
-}
-
 QVariant toExportValue(const QVariant &value)
 {
     int type = value.userType();
 
-    if (type == QVariant::Color)
-        return colorToString(value.value<QColor>());
-
-    if (type == filePathTypeId()) {
-        const FilePath filePath = value.value<FilePath>();
-        return filePath.url.toString(QUrl::PreferLocalFile);
+    if (type == QVariant::Color) {
+        const QColor color = value.value<QColor>();
+        return color.isValid() ? color.name(QColor::HexArgb) : QString();
     }
 
+    if (type == filePathTypeId())
+        return FilePath::toString(value.value<FilePath>());
+
+    if (type == objectRefTypeId())
+        return ObjectRef::toInt(value.value<ObjectRef>());
+
     return value;
 }
 
@@ -170,12 +186,11 @@ QVariant fromExportValue(const QVariant &value, int type)
     if (value.userType() == type)
         return value;
 
-    if (type == filePathTypeId()) {
-        QUrl url(value.toString());
-        if (url.isRelative())
-            url = QUrl::fromLocalFile(value.toString());
-        return QVariant::fromValue(FilePath { url });
-    }
+    if (type == filePathTypeId())
+        return QVariant::fromValue(FilePath::fromString(value.toString()));
+
+    if (type == objectRefTypeId())
+        return QVariant::fromValue(ObjectRef::fromInt(value.toInt()));
 
     QVariant variant(value);
     variant.convert(type);
@@ -202,4 +217,13 @@ QVariant fromExportValue(const QVariant &value, int type, const QDir &dir)
     return fromExportValue(value, type);
 }
 
+void initializeMetatypes()
+{
+    QMetaType::registerConverter<ObjectRef, int>(&ObjectRef::toInt);
+    QMetaType::registerConverter<int, ObjectRef>(&ObjectRef::fromInt);
+
+    QMetaType::registerConverter<FilePath, QString>(&FilePath::toString);
+    QMetaType::registerConverter<QString, FilePath>(&FilePath::fromString);
+}
+
 } // namespace Tiled
