@@ -40,7 +40,6 @@ Tile::Tile(int id, Tileset *tileset):
     mImageStatus(LoadingReady),
     mTerrain(-1),
     mProbability(1.0),
-    mObjectGroup(nullptr),
     mCurrentFrameIndex(0),
     mUnusedTime(0)
 {}
@@ -53,14 +52,12 @@ Tile::Tile(const QPixmap &image, int id, Tileset *tileset):
     mImageStatus(image.isNull() ? LoadingError : LoadingReady),
     mTerrain(-1),
     mProbability(1.0),
-    mObjectGroup(nullptr),
     mCurrentFrameIndex(0),
     mUnusedTime(0)
 {}
 
 Tile::~Tile()
 {
-    delete mObjectGroup;
 }
 
 /**
@@ -119,29 +116,24 @@ void Tile::setTerrain(unsigned terrain)
  * The Tile takes ownership over the ObjectGroup and it can't also be part of
  * a map.
  */
-void Tile::setObjectGroup(ObjectGroup *objectGroup)
+void Tile::setObjectGroup(std::unique_ptr<ObjectGroup> objectGroup)
 {
     Q_ASSERT(!objectGroup || !objectGroup->map());
 
     if (mObjectGroup == objectGroup)
         return;
 
-    delete mObjectGroup;
-    mObjectGroup = objectGroup;
+    mObjectGroup = std::move(objectGroup);
 }
 
 /**
  * Swaps the object group of this tile with \a objectGroup. The tile releases
  * ownership over its existing object group and takes ownership over the new
  * one.
- *
- * @return The previous object group referenced by this tile.
  */
-ObjectGroup *Tile::swapObjectGroup(ObjectGroup *objectGroup)
+void Tile::swapObjectGroup(std::unique_ptr<ObjectGroup> &objectGroup)
 {
-    ObjectGroup *previousObjectGroup = mObjectGroup;
-    mObjectGroup = objectGroup;
-    return previousObjectGroup;
+    std::swap(mObjectGroup, objectGroup);
 }
 
 /**
@@ -205,11 +197,13 @@ Tile *Tile::clone(Tileset *tileset) const
     c->setProperties(properties());
 
     c->mImageSource = mImageSource;
+    c->mImageStatus = mImageStatus;
+    c->mType = mType;
     c->mTerrain = mTerrain;
     c->mProbability = mProbability;
 
     if (mObjectGroup)
-        c->mObjectGroup = mObjectGroup->clone();
+        c->mObjectGroup.reset(mObjectGroup->clone());
 
     c->mFrames = mFrames;
     c->mCurrentFrameIndex = mCurrentFrameIndex;
