@@ -35,6 +35,7 @@
 class QEvent;
 class QKeyEvent;
 class QToolBar;
+class QUndoStack;
 
 namespace Tiled {
 
@@ -98,6 +99,9 @@ public:
     bool isEnabled() const;
     void setEnabled(bool enabled);
 
+    bool isVisible() const;
+    void setVisible(bool visible);
+
     ToolManager *toolManager() const;
     Tile *tile() const;
     ObjectTemplate *objectTemplate() const;
@@ -167,6 +171,11 @@ public:
 
     void setMapDocument(MapDocument *mapDocument);
 
+    /**
+     * override to use a different undo stack than the one from the document.
+     */
+    virtual QUndoStack* undoStack() { return nullptr; }
+
 protected:
     virtual void changeEvent(const ChangeEvent &event);
 
@@ -198,6 +207,7 @@ signals:
     void statusInfoChanged(const QString &statusInfo);
     void cursorChanged(const QCursor &cursor);
     void enabledChanged(bool enabled);
+    void visibleChanged(bool visible);
 
 private:
     friend class ToolManager;
@@ -208,10 +218,11 @@ private:
     QString mStatusInfo;
     QCursor mCursor;
     Id mId;
-    bool mEnabled;
+    bool mEnabled = false;
+    bool mVisible = true;
 
-    ToolManager *mToolManager;
-    MapDocument *mMapDocument;
+    ToolManager *mToolManager = nullptr;
+    MapDocument *mMapDocument = nullptr;
 };
 
 
@@ -256,6 +267,11 @@ inline bool AbstractTool::isEnabled() const
     return mEnabled;
 }
 
+inline bool AbstractTool::isVisible() const
+{
+    return mVisible;
+}
+
 /**
  * Returns the ToolManager with which this tool is registered, if any.
  */
