@@ -18,19 +18,15 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef ADDREMOVETILES_H
-#define ADDREMOVETILES_H
+#pragma once
 
 #include <QUndoCommand>
 
 namespace Tiled {
 
 class Tile;
-class Tileset;
 
-namespace Internal {
-
-class MapDocument;
+class TilesetDocument;
 
 /**
  * Abstract base class for AddTiles and RemoveTiles.
@@ -38,8 +34,7 @@ class MapDocument;
 class AddRemoveTiles : public QUndoCommand
 {
 public:
-    AddRemoveTiles(MapDocument *mapDocument,
-                   Tileset *tileset,
+    AddRemoveTiles(TilesetDocument *tilesetDocument,
                    const QList<Tile*> &tiles,
                    bool add);
 
@@ -50,8 +45,7 @@ protected:
     void removeTiles();
 
 private:
-    MapDocument *mMapDocument;
-    Tileset *mTileset;
+    TilesetDocument *mTilesetDocument;
     QList<Tile*> mTiles;
     bool mTilesAdded;
 };
@@ -62,8 +56,7 @@ private:
 class AddTiles : public AddRemoveTiles
 {
 public:
-    AddTiles(MapDocument *mapDocument,
-             Tileset *tileset,
+    AddTiles(TilesetDocument *tilesetDocument,
              const QList<Tile*> &tiles);
 
     void undo() override
@@ -79,8 +72,7 @@ public:
 class RemoveTiles : public AddRemoveTiles
 {
 public:
-    RemoveTiles(MapDocument *mapDocument,
-                Tileset *tileset,
+    RemoveTiles(TilesetDocument *tilesetDocument,
                 const QList<Tile *> &tiles);
 
     void undo() override
@@ -90,7 +82,4 @@ public:
     { removeTiles(); }
 };
 
-} // namespace Internal
 } // namespace Tiled
-
-#endif // TILED_INTERNAL_ADDREMOVETILES_H
