@@ -20,6 +20,8 @@
 
 #pragma once
 
+#include "session.h"
+
 #include <QObject>
 #include <QRegion>
 #include <QString>
@@ -61,6 +63,8 @@ public:
     void autoMap();
     void autoMapRegion(const QRegion &region);
 
+    static SessionOption<bool> automappingWhileDrawing;
+
 signals:
     /**
      * This signal is emitted after automapping was done and an error occurred.
