@@ -25,6 +25,7 @@
 #include <QPointF>
 #include <QSize>
 #include <QUndoCommand>
+#include <QColor>
 
 namespace Tiled {
 
@@ -120,6 +121,29 @@ private:
     qreal mNewOpacity;
 };
 
+class SetLayerTintColor : public QUndoCommand
+{
+public:
+    SetLayerTintColor(Document *document,
+                    Layer *layer,
+                    QColor tintColor);
+
+    void undo() override { setTintColor(mOldTintColor); }
+    void redo() override { setTintColor(mNewTintColor); }
+
+    int id() const override { return Cmd_ChangeLayerTintColor; }
+
+    bool mergeWith(const QUndoCommand *other) override;
+
+private:
+    void setTintColor(QColor tintColor);
+
+    Document *mDocument;
+    Layer *mLayer;
+    QColor mOldTintColor;
+    QColor mNewTintColor;
+};
+
 /**
  * Used for changing the layer offset.
  */
