@@ -21,13 +21,12 @@
 #pragma once
 
 #include <QUndoCommand>
+#include <QUrl>
 
 namespace Tiled {
 
 class Tile;
 
-namespace Internal {
-
 class TilesetDocument;
 
 class ChangeTileImageSource : public QUndoCommand
@@ -35,19 +34,18 @@ class ChangeTileImageSource : public QUndoCommand
 public:
     ChangeTileImageSource(TilesetDocument *tilesetDocument,
                           Tile *tile,
-                          const QString &imageSource);
+                          const QUrl &imageSource);
 
     void undo() { apply(mOldImageSource); }
     void redo() { apply(mNewImageSource); }
 
 private:
-    void apply(const QString &imageSource);
+    void apply(const QUrl &imageSource);
 
     TilesetDocument *mTilesetDocument;
     Tile *mTile;
-    QString mOldImageSource;
-    QString mNewImageSource;
+    QUrl mOldImageSource;
+    QUrl mNewImageSource;
 };
 
-} // namespace Internal
 } // namespace Tiled
