@@ -28,6 +28,7 @@
 #include <QList>
 #include <QObject>
 #include <QPointF>
+#include <QPointer>
 #include <QVector>
 
 class QTabWidget;
@@ -45,11 +46,13 @@ class BrokenLinksWidget;
 class Document;
 class Editor;
 class FileChangedWarning;
+class MainWindow;
 class MapDocument;
 class MapEditor;
 class MapView;
 class TilesetDocument;
 class TilesetDocumentsModel;
+class WorldDocument;
 
 /**
  * This class controls the open documents.
@@ -63,15 +66,17 @@ class DocumentManager : public QObject
     DocumentManager(QObject *parent = nullptr);
     ~DocumentManager() override;
 
+    friend class MainWindow;
+
 public:
     static DocumentManager *instance();
-    static void deleteInstance();
+    static DocumentManager *maybeInstance();
 
     QWidget *widget() const;
 
     void setEditor(Document::DocumentType documentType, Editor *editor);
     Editor *editor(Document::DocumentType documentType) const;
-    void deleteEditor(Document::DocumentType documentType);
+    void deleteEditors();
     QList<Editor*> editors() const;
 
     Editor *currentEditor() const;
@@ -90,8 +95,10 @@ public:
     int findDocument(Document *document) const;
 
     void switchToDocument(int index);
+    bool switchToDocument(const QString &fileName);
     bool switchToDocument(Document *document);
     void switchToDocument(MapDocument *mapDocument, QPointF viewCenter, qreal scale);
+    void switchToDocumentAndHandleSimiliarTileset(MapDocument *mapDocument, QPointF viewCenter, qreal scale);
 
     void addDocument(const DocumentPtr &document);
     void insertDocument(int index, const DocumentPtr &document);
@@ -129,7 +136,11 @@ public:
 
     void abortMultiDocumentClose();
 
-    bool eventFilter(QObject *object, QEvent *event) override;
+    WorldDocument *ensureWorldDocument(const QString &fileName);
+    bool isAnyWorldModified() const;
+    bool isWorldModified(const QString &fileName) const;
+
+    QString fileDialogStartLocation() const;
 
 signals:
     void documentCreated(Document *document);
@@ -159,6 +170,8 @@ signals:
      */
     void documentAboutToClose(Document *document);
 
+    void currentEditorChanged(Editor *editor);
+
     /**
      * Emitted when an error occurred while reloading the map.
      */
@@ -176,6 +189,8 @@ public slots:
     void saveFile();
 
 private:
+    void onWorldUnloaded(const QString &worldFile);
+
     void currentIndexChanged();
     void fileNameChanged(const QString &fileName,
                          const QString &oldFileName);
@@ -189,6 +204,7 @@ private:
 
     void tilesetNameChanged(Tileset *tileset);
 
+    void filesChanged(const QStringList &fileNames);
     void fileChanged(const QString &fileName);
     void hideChangedWarning();
 
@@ -199,13 +215,17 @@ private:
     void addToTilesetDocument(const SharedTileset &tileset, MapDocument *mapDocument);
     void removeFromTilesetDocument(const SharedTileset &tileset, MapDocument *mapDocument);
 
+    void updateSession() const;
+
     MapDocument *openMapFile(const QString &path);
     TilesetDocument *openTilesetFile(const QString &path);
 
     QVector<DocumentPtr> mDocuments;
+    QMap<QString, WorldDocument*> mWorldDocuments;
     TilesetDocumentsModel *mTilesetDocumentsModel;
 
-    QWidget *mWidget;
+    // Pointer becomes null when deleted as part of the UI, to prevent double-deletion
+    QPointer<QWidget> mWidget;
     QWidget *mNoEditorWidget;
     QTabBar *mTabBar;
     FileChangedWarning *mFileChangedWarning;
