@@ -61,6 +61,8 @@ public:
 
     virtual QList<QToolBar*> toolBars() const = 0;
     virtual QList<QDockWidget*> dockWidgets() const = 0;
+    virtual QList<QWidget*> statusBarWidgets() const = 0;
+    virtual QList<QWidget*> permanentStatusBarWidgets() const = 0;
 
     virtual StandardActions enabledStandardActions() const = 0;
     virtual void performStandardAction(StandardAction action) = 0;
