@@ -21,31 +21,39 @@
 #pragma once
 
 #include <QLatin1String>
+#include <QMetaType>
 
 namespace Tiled {
-namespace Internal {
 
 class Id
 {
 public:
+    Id() : mId(0) {}
     Id(const char *name);
+    Id(const QByteArray &name);
 
     QByteArray name() const;
+    QString toString() const;
+    bool isNull() const { return mId == 0; }
+
+    explicit operator bool() const { return !isNull(); }
 
     bool operator==(Id id) const { return mId == id.mId; }
     bool operator!=(Id id) const { return mId != id.mId; }
+    bool operator<(Id id) const { return name() < id.name(); }
 
 private:
     uint mId;
 
-    friend uint qHash(Id id);
+    friend uint qHash(Id id) Q_DECL_NOTHROW;
 };
 
 
-inline uint qHash(Id id)
+inline uint qHash(Id id) Q_DECL_NOTHROW
 {
     return id.mId;
 }
 
-} // namespace Internal
 } // namespace Tiled
+
+Q_DECLARE_METATYPE(Tiled::Id)
