@@ -29,6 +29,8 @@
 #include "tiled.h"
 #include "tileset.h"
 
+#include <memory>
+
 class QAction;
 class QComboBox;
 class QLabel;
@@ -49,7 +51,6 @@ class EditPolygonTool;
 class LayerDock;
 class MapDocument;
 class MapView;
-class MapsDock;
 class MiniMapDock;
 class ObjectsDock;
 class PropertiesDock;
@@ -83,6 +84,7 @@ public:
     ~MapEditor() override;
 
     TilesetDock *tilesetDock() const { return mTilesetDock; }
+    TemplatesDock *templatesDock() const { return mTemplatesDock; }
 
     void saveState() override;
     void restoreState() override;
@@ -97,6 +99,8 @@ public:
 
     QList<QToolBar *> toolBars() const override;
     QList<QDockWidget *> dockWidgets() const override;
+    QList<QWidget*> statusBarWidgets() const override;
+    QList<QWidget*> permanentStatusBarWidgets() const override;
 
     StandardActions enabledStandardActions() const override;
     void performStandardAction(StandardAction action) override;
@@ -107,9 +111,8 @@ public:
     MapView *currentMapView() const;
     Zoomable *zoomable() const override;
 
-    void saveDocumentState(MapDocument *mapDocument);
-
-    void showMessage(const QString &text, int timeout = 0);
+    void saveDocumentState(MapDocument *mapDocument) const;
+    void restoreDocumentState(MapDocument *mapDocument) const;
 
     void setCurrentTileset(const SharedTileset &tileset);
     SharedTileset currentTileset() const;
@@ -122,8 +125,12 @@ public:
     QAction *actionSelectNextTileset() const;
     QAction *actionSelectPreviousTileset() const;
 
+    AbstractTool *selectedTool() const;
+
 private:
     void setSelectedTool(AbstractTool *tool);
+    void currentDocumentChanged(Document *document);
+    void updateActiveUndoStack();
 
     void paste(ClipboardManager::PasteFlags flags);
 
@@ -163,7 +170,6 @@ private:
     MapDocument *mCurrentMapDocument;
 
     PropertiesDock *mPropertiesDock;
-    MapsDock *mMapsDock;
     UndoDock *mUndoDock;
     ObjectsDock *mObjectsDock;
     TemplatesDock *mTemplatesDock;
@@ -173,13 +179,13 @@ private:
     MiniMapDock* mMiniMapDock;
     TileStampsDock *mTileStampsDock;
 
-    TreeViewComboBox *mLayerComboBox;
+    std::unique_ptr<TreeViewComboBox> mLayerComboBox;
     ComboBoxProxyModel *mComboBoxProxyModel;
     ReversingProxyModel *mReversingProxyModel;
 
     Zoomable *mZoomable;
-    QComboBox *mZoomComboBox;
-    QLabel *mStatusInfoLabel;
+    std::unique_ptr<QComboBox> mZoomComboBox;
+    std::unique_ptr<QLabel> mStatusInfoLabel;
 
     StampBrush *mStampBrush;
     BucketFillTool *mBucketFillTool;
@@ -196,8 +202,6 @@ private:
     MapView *mViewWithTool;
 
     TileStampManager *mTileStampManager;
-
-    QVariantMap mMapStates;
 };
 
 
