@@ -38,6 +38,7 @@ class TileLayer;
 class Tileset;
 
 class BorderItem;
+class LayerChangeEvent;
 class LayerItem;
 class MapObjectItem;
 class ObjectSelectionItem;
@@ -64,6 +65,9 @@ public:
             QGraphicsItem *parent = nullptr);
     ~MapItem() override;
 
+    enum { Type = UserType + 3 };
+    int type() const override { return Type; }
+
     MapDocument *mapDocument() const;
 
     void setDisplayMode(DisplayMode displayMode);
@@ -84,6 +88,8 @@ protected:
     void mouseReleaseEvent(QGraphicsSceneMouseEvent *event) override;
 
 private:
+    bool isWorldToolSelected() const;
+
     /**
      * Repaints the specified \a region of the given \a tileLayer. The region
      * is in tile coordinates.
@@ -96,7 +102,8 @@ private:
 
     void layerAdded(Layer *layer);
     void layerRemoved(Layer *layer);
-    void layerChanged(Layer *layer);
+    void layerChanged(const LayerChangeEvent &change);
+    void layerTintColorChanged(Layer *layer);
 
     void imageLayerChanged(ImageLayer *imageLayer);
 
@@ -107,7 +114,7 @@ private:
     void tilesetReplaced(int index, Tileset *tileset);
 
     void objectsInserted(ObjectGroup *objectGroup, int first, int last);
-    void deleteObjectItems(const QList<MapObject*> &objects);
+    void deleteObjectItem(MapObject *object);
     void syncObjectItems(const QList<MapObject*> &objects);
     void objectsIndexChanged(ObjectGroup *objectGroup, int first, int last);
 
