@@ -40,6 +40,7 @@
 #include "worldmanager.h"
 
 #include <QApplication>
+#include <QFileInfo>
 #include <QGraphicsSceneMouseEvent>
 #include <QKeyEvent>
 #include <QMimeData>
@@ -89,7 +90,7 @@ void MapScene::setMapDocument(MapDocument *mapDocument)
     if (mMapDocument) {
         connect(mMapDocument, &MapDocument::mapChanged,
                 this, &MapScene::mapChanged);
-        connect(mMapDocument, &MapDocument::tilesetTileOffsetChanged,
+        connect(mMapDocument, &MapDocument::tilesetTilePositioningChanged,
                 this, [this] { update(); });
         connect(mMapDocument, &MapDocument::tileImageSourceChanged,
                 this, [this] { update(); });
@@ -208,6 +209,8 @@ void MapScene::refreshScene()
         setBackgroundBrush(map->backgroundColor());
     else
         setBackgroundBrush(mDefaultBackgroundColor);
+
+    emit sceneRefreshed();
 }
 
 void MapScene::updateDefaultBackgroundColor()
@@ -366,16 +369,20 @@ void MapScene::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *mouseEvent)
 
 static const ObjectTemplate *readObjectTemplate(const QMimeData *mimeData)
 {
-    if (!mimeData->hasFormat(QLatin1String(TEMPLATES_MIMETYPE)))
+    const auto urls = mimeData->urls();
+    if (urls.size() != 1)
         return nullptr;
 
-    QByteArray encodedData = mimeData->data(QLatin1String(TEMPLATES_MIMETYPE));
-    QDataStream stream(&encodedData, QIODevice::ReadOnly);
+    const QString fileName = urls.first().toLocalFile();
+    if (fileName.isEmpty())
+        return nullptr;
 
-    QString fileName;
-    stream >> fileName;
+    const QFileInfo info(fileName);
+    if (info.isDir())
+        return nullptr;
 
-    return TemplateManager::instance()->findObjectTemplate(fileName);
+    auto objectTemplate = TemplateManager::instance()->loadObjectTemplate(info.absoluteFilePath());
+    return objectTemplate->object() ? objectTemplate : nullptr;
 }
 
 /**
