@@ -66,9 +66,13 @@ public:
 
     void setSelectedTool(AbstractTool *tool);
 
+    MapItem *mapItem(MapDocument *mapDocument) const;
+
 signals:
     void mapDocumentChanged(MapDocument *mapDocument);
 
+    void sceneRefreshed();
+
 protected:
     bool event(QEvent *event) override;
 
@@ -117,4 +121,12 @@ inline MapDocument *MapScene::mapDocument() const
     return mMapDocument;
 }
 
+/**
+ * Returns the map item displaying the given map, if any.
+ */
+inline MapItem *MapScene::mapItem(MapDocument *mapDocument) const
+{
+    return mMapItems.value(mapDocument);
+}
+
 } // namespace Tiled
