@@ -122,13 +122,26 @@ void MapView::setScale(qreal scale)
 
 void MapView::fitMapInView()
 {
-    const QRectF rect = mapScene()->mapBoundingRect();
+    MapScene* scene = mapScene();
+    if (!scene)
+        return;
+
+    const QRectF rect = scene->mapBoundingRect();
     if (rect.isEmpty())
         return;
 
-    // Scale and center map to fit in view
+    // Scale and center map to fit in view, while avoiding to go below the
+    // minimum scale. For extremely large maps, avoid putting more than about
+    // 256 * 256 tiles within the view for performance reasons.
+    qreal desiredScale = std::min(width() / rect.width(),
+                                  height() / rect.height()) * 0.95;
+
+    auto tileSize = mMapDocument->map()->tileSize();
+    qreal scale256 = std::min(width() / (256.0 * tileSize.width()),
+                              height() / (256.0 * tileSize.height()));
+
     centerOn(rect.center());
-    setScale(std::min(width() / rect.width(), height() / rect.height()) * 0.95);
+    setScale(std::max(std::max(desiredScale, scale256), 0.015625));
 }
 
 void MapView::adjustScale(qreal scale)
@@ -250,6 +263,14 @@ void MapView::setHandScrolling(bool handScrolling)
  */
 void MapView::forceCenterOn(const QPointF &pos)
 {
+    // Let's wait until the initial paint event before we position the view,
+    // otherwise layout changes may still affect the position.
+    if (!mViewInitialized) {
+        mInitialCenterPos = pos;
+        mHasInitialCenterPos = true;
+        return;
+    }
+
     // This is only to make it update QGraphicsViewPrivate::lastCenterPoint,
     // just in case this is important.
     QGraphicsView::centerOn(pos);
@@ -306,14 +327,18 @@ bool MapView::event(QEvent *e)
     return QGraphicsView::event(e);
 }
 
-void MapView::showEvent(QShowEvent *event)
+void MapView::paintEvent(QPaintEvent *event)
 {
     if (!mViewInitialized) {
-        fitMapInView();
         mViewInitialized = true;
+
+        if (mHasInitialCenterPos)
+            forceCenterOn(mInitialCenterPos);
+        else
+            fitMapInView();
     }
 
-    QGraphicsView::showEvent(event);
+    QGraphicsView::paintEvent(event);
 }
 
 void MapView::hideEvent(QHideEvent *event)
