@@ -61,8 +61,6 @@ public:
     MapView(QWidget *parent = nullptr, Mode mode = StaticContents);
     ~MapView() override;
 
-    void setViewInitialized();
-
     void setScene(MapScene *scene);
     MapScene *mapScene() const;
 
@@ -77,14 +75,14 @@ public:
     void setHandScrolling(bool handScrolling);
 
     using QGraphicsView::centerOn;
-    Q_INVOKABLE void centerOn(qreal x, qreal y) { forceCenterOn(QPointF(x, y)); };
+    Q_INVOKABLE void centerOn(qreal x, qreal y) { forceCenterOn(QPointF(x, y)); }
 
     void forceCenterOn(const QPointF &pos);
 
 protected:
     bool event(QEvent *event) override;
 
-    void showEvent(QShowEvent *) override;
+    void paintEvent(QPaintEvent *event) override;
     void hideEvent(QHideEvent *) override;
     void resizeEvent(QResizeEvent *event) override;
 
@@ -119,16 +117,12 @@ private:
     QPointF mLastMouseScenePos;
     bool mHandScrolling = false;
     bool mViewInitialized = false;
+    bool mHasInitialCenterPos = false;
+    QPointF mInitialCenterPos;
     Mode mMode;
     Zoomable *mZoomable;
 };
 
-
-inline void MapView::setViewInitialized()
-{
-    mViewInitialized = true;
-}
-
 } // namespace Tiled
 
 Q_DECLARE_METATYPE(Tiled::MapView*)
