@@ -22,12 +22,12 @@
 
 #include <QWidget>
 
-namespace Tiled {
-
 namespace Ui {
 class NoEditorWidget;
 }
 
+namespace Tiled {
+
 class NoEditorWidget : public QWidget
 {
     Q_OBJECT
@@ -44,6 +44,12 @@ private:
     void newTileset();
     void openFile();
 
+    void retranslateUi();
+
+    void updateRecentProjectsMenu();
+
+    void adjustToStyle();
+
     Ui::NoEditorWidget *ui;
 };
 
