@@ -28,12 +28,15 @@
 #include "mapobjectitem.h"
 #include "maprenderer.h"
 #include "objectgroup.h"
+#include "objectreferenceitem.h"
 #include "preferences.h"
 #include "tile.h"
 #include "utils.h"
+#include "variantpropertymanager.h"
 
 #include <QGuiApplication>
 #include <QTimerEvent>
+#include <QVector2D>
 
 #include "qtcompat_p.h"
 
@@ -142,33 +145,14 @@ void MapObjectOutline::timerEvent(QTimerEvent *event)
 }
 
 
-class MapObjectLabel : public QGraphicsItem
+MapObjectLabel::MapObjectLabel(const MapObject *object, QGraphicsItem *parent)
+    : QGraphicsItem(parent)
+    , mObject(object)
+    , mColor(mObject->effectiveColor())
 {
-public:
-    MapObjectLabel(MapObject *object, QGraphicsItem *parent = nullptr)
-        : QGraphicsItem(parent)
-        , mObject(object)
-        , mColor(mObject->effectiveColor())
-    {
-        setFlags(QGraphicsItem::ItemIgnoresTransformations |
-                 QGraphicsItem::ItemIgnoresParentOpacity);
-    }
-
-    MapObject *mapObject() const { return mObject; }
-    void syncWithMapObject(const MapRenderer &renderer);
-    void updateColor();
-
-    QRectF boundingRect() const override;
-    void paint(QPainter *painter,
-               const QStyleOptionGraphicsItem *,
-               QWidget *) override;
-
-private:
-    QRectF mBoundingRect;
-    QPointF mTextPos;
-    MapObject *mObject;
-    QColor mColor;
-};
+    setFlags(QGraphicsItem::ItemIgnoresTransformations |
+             QGraphicsItem::ItemIgnoresParentOpacity);
+}
 
 void MapObjectLabel::syncWithMapObject(const MapRenderer &renderer)
 {
@@ -252,6 +236,14 @@ ObjectSelectionItem::ObjectSelectionItem(MapDocument *mapDocument,
 
     connect(mapDocument, &Document::changed,
             this, &ObjectSelectionItem::changeEvent);
+    connect(mapDocument, &Document::propertyAdded,
+            this, &ObjectSelectionItem::propertyAdded);
+    connect(mapDocument, &Document::propertyRemoved,
+            this, &ObjectSelectionItem::propertyRemoved);
+    connect(mapDocument, &Document::propertyChanged,
+            this, &ObjectSelectionItem::propertyChanged);
+    connect(mapDocument, &Document::propertiesChanged,
+            this, &ObjectSelectionItem::propertiesChanged);
 
     connect(mapDocument, &MapDocument::selectedObjectsChanged,
             this, &ObjectSelectionItem::selectedObjectsChanged);
@@ -268,8 +260,8 @@ ObjectSelectionItem::ObjectSelectionItem(MapDocument *mapDocument,
     connect(mapDocument, &MapDocument::hoveredMapObjectChanged,
             this, &ObjectSelectionItem::hoveredMapObjectChanged);
 
-    connect(mapDocument, &MapDocument::tilesetTileOffsetChanged,
-            this, &ObjectSelectionItem::tilesetTileOffsetChanged);
+    connect(mapDocument, &MapDocument::tilesetTilePositioningChanged,
+            this, &ObjectSelectionItem::tilesetTilePositioningChanged);
 
     connect(mapDocument, &MapDocument::tileTypeChanged,
             this, &ObjectSelectionItem::tileTypeChanged);
@@ -278,18 +270,30 @@ ObjectSelectionItem::ObjectSelectionItem(MapDocument *mapDocument,
 
     connect(prefs, &Preferences::objectLabelVisibilityChanged,
             this, &ObjectSelectionItem::objectLabelVisibilityChanged);
+    connect(prefs, &Preferences::showObjectReferencesChanged,
+            this, &ObjectSelectionItem::showObjectReferencesChanged);
+    connect(prefs, &Preferences::objectLineWidthChanged,
+            this, &ObjectSelectionItem::objectLineWidthChanged);
 
     connect(prefs, &Preferences::objectTypesChanged,
-            this, &ObjectSelectionItem::updateObjectLabelColors);
+            this, &ObjectSelectionItem::updateItemColors);
 
     if (objectLabelVisibility() == Preferences::AllObjectLabels)
         addRemoveObjectLabels();
+
+    if (Preferences::instance()->showObjectReferences())
+        addRemoveObjectReferences();
 }
 
 ObjectSelectionItem::~ObjectSelectionItem()
 {
 }
 
+const MapRenderer &ObjectSelectionItem::mapRenderer() const
+{
+    return *mMapDocument->renderer();
+}
+
 void ObjectSelectionItem::changeEvent(const ChangeEvent &event)
 {
     switch (event.type) {
@@ -307,13 +311,60 @@ void ObjectSelectionItem::changeEvent(const ChangeEvent &event)
         break;
     case ChangeEvent::ObjectGroupChanged:
         if (static_cast<const ObjectGroupChangeEvent&>(event).properties & ObjectGroupChangeEvent::ColorProperty)
-            updateObjectLabelColors();
+            updateItemColors();
         break;
     default:
         break;
     }
 }
 
+void ObjectSelectionItem::propertyAdded(Object *object, const QString &name)
+{
+    if (object->typeId() != Object::MapObjectType)
+        return;
+    if (!Preferences::instance()->showObjectReferences())
+        return;
+    if (object->property(name).userType() != objectRefTypeId())
+        return;
+
+    addRemoveObjectReferences(static_cast<MapObject*>(object));
+}
+
+void ObjectSelectionItem::propertyRemoved(Object *object, const QString &name)
+{
+    Q_UNUSED(name)
+
+    if (object->typeId() != Object::MapObjectType)
+        return;
+    if (!mReferencesBySourceObject.contains(static_cast<MapObject*>(object)))
+        return;
+
+    addRemoveObjectReferences(static_cast<MapObject*>(object));
+}
+
+void ObjectSelectionItem::propertyChanged(Object *object, const QString &name)
+{
+    if (object->typeId() != Object::MapObjectType)
+        return;
+    if (!Preferences::instance()->showObjectReferences())
+        return;
+    if (object->property(name).userType() != objectRefTypeId() &&
+            !mReferencesBySourceObject.contains(static_cast<MapObject*>(object)))
+        return;
+
+    addRemoveObjectReferences(static_cast<MapObject*>(object));
+}
+
+void ObjectSelectionItem::propertiesChanged(Object *object)
+{
+    if (object->typeId() != Object::MapObjectType)
+        return;
+    if (!Preferences::instance()->showObjectReferences())
+        return;
+
+    addRemoveObjectReferences(static_cast<MapObject*>(object));
+}
+
 void ObjectSelectionItem::selectedObjectsChanged()
 {
     addRemoveObjectLabels();
@@ -402,6 +453,9 @@ void ObjectSelectionItem::layerAdded(Layer *layer)
             mObjectLabels.insert(object, labelItem);
         }
     }
+
+    if (Preferences::instance()->showObjectReferences())
+        addRemoveObjectReferences();
 }
 
 void ObjectSelectionItem::layerAboutToBeRemoved(GroupLayer *parentLayer, int index)
@@ -447,17 +501,31 @@ void ObjectSelectionItem::syncOverlayItems(const QList<MapObject*> &objects)
     for (MapObject *object : objects) {
         if (MapObjectOutline *outlineItem = mObjectOutlines.value(object))
             outlineItem->syncWithMapObject(renderer);
+
         if (MapObjectLabel *labelItem = mObjectLabels.value(object))
             labelItem->syncWithMapObject(renderer);
+
+        const auto sourceItems = mReferencesBySourceObject.value(object);
+        for (auto item : sourceItems)
+            item->syncWithSourceObject(renderer);
+
+        const auto targetItems = mReferencesByTargetObject.value(object);
+        for (auto item : targetItems)
+            item->syncWithTargetObject(renderer);
+
         if (mHoveredMapObjectItem && mHoveredMapObjectItem->mapObject() == object)
             mHoveredMapObjectItem->syncWithMapObject();
     }
 }
 
-void ObjectSelectionItem::updateObjectLabelColors()
+void ObjectSelectionItem::updateItemColors() const
 {
-    for (MapObjectLabel *label : qAsConst(mObjectLabels))
+    for (MapObjectLabel *label : mObjectLabels)
         label->updateColor();
+
+    for (const auto &referenceItems : mReferencesBySourceObject)
+        for (ObjectReferenceItem *item : referenceItems)
+            item->updateColor();
 }
 
 void ObjectSelectionItem::objectsAdded(const QList<MapObject *> &objects)
@@ -473,6 +541,13 @@ void ObjectSelectionItem::objectsAdded(const QList<MapObject *> &objects)
             mObjectLabels.insert(object, labelItem);
         }
     }
+
+    // This could be a tad slow if there are many objects, but currently it is
+    // necessary because the list of added objects could include target
+    // objects. To optimize this, we could instantiate reference items also if
+    // the target was not found and only try to repair those here.
+    if (Preferences::instance()->showObjectReferences())
+        addRemoveObjectReferences();
 }
 
 void ObjectSelectionItem::objectsAboutToBeRemoved(const QList<MapObject *> &objects)
@@ -480,11 +555,27 @@ void ObjectSelectionItem::objectsAboutToBeRemoved(const QList<MapObject *> &obje
     if (objectLabelVisibility() == Preferences::AllObjectLabels)
         for (MapObject *object : objects)
             delete mObjectLabels.take(object);
+
+    for (MapObject *object : objects) {
+        auto it = mReferencesBySourceObject.find(object);
+        if (it != mReferencesBySourceObject.end()) {
+            QList<ObjectReferenceItem*> &items = *it;
+            for (auto item : items) {
+                auto &itemsByTarget = mReferencesByTargetObject[item->targetObject()];
+                itemsByTarget.removeOne(item);
+                if (itemsByTarget.isEmpty())
+                    mReferencesByTargetObject.remove(item->targetObject());
+
+                delete item;
+            }
+            mReferencesBySourceObject.erase(it);
+        }
+    }
 }
 
-void ObjectSelectionItem::tilesetTileOffsetChanged(Tileset *tileset)
+void ObjectSelectionItem::tilesetTilePositioningChanged(Tileset *tileset)
 {
-    // Tile offset affects the position of selection outlines and labels
+    // Tile offset and alignment affect the position of selection outlines and labels
     const MapRenderer &renderer = *mMapDocument->renderer();
 
     for (MapObjectLabel *label : qAsConst(mObjectLabels))
@@ -501,12 +592,22 @@ void ObjectSelectionItem::tilesetTileOffsetChanged(Tileset *tileset)
 
 void ObjectSelectionItem::tileTypeChanged(Tile *tile)
 {
-    for (MapObjectLabel *label : qAsConst(mObjectLabels)) {
-        MapObject *object = label->mapObject();
-        if (object->type().isEmpty()) {
-            const auto &cell = object->cell();
-            if (cell.tileset() == tile->tileset() && cell.tileId() == tile->id())
-                label->updateColor();
+    auto isObjectAffected = [tile] (const MapObject *object) -> bool {
+        if (!object->type().isEmpty())
+            return false;
+
+        const auto &cell = object->cell();
+        return cell.tileset() == tile->tileset() && cell.tileId() == tile->id();
+    };
+
+    for (MapObjectLabel *label : qAsConst(mObjectLabels))
+        if (isObjectAffected(label->mapObject()))
+            label->updateColor();
+
+    for (auto it = mReferencesByTargetObject.constBegin(), it_end = mReferencesByTargetObject.constEnd(); it != it_end; ++it) {
+        if (isObjectAffected(it.key())) {
+            for (ObjectReferenceItem *item : it.value())
+                item->updateColor();
         }
     }
 }
@@ -516,6 +617,19 @@ void ObjectSelectionItem::objectLabelVisibilityChanged()
     addRemoveObjectLabels();
 }
 
+void ObjectSelectionItem::showObjectReferencesChanged()
+{
+    addRemoveObjectReferences();
+}
+
+void ObjectSelectionItem::objectLineWidthChanged()
+{
+    // Object reference items should redraw when line width is changed
+    for (const auto &items : mReferencesBySourceObject)
+        for (ObjectReferenceItem *item : items)
+            item->update();
+}
+
 void ObjectSelectionItem::addRemoveObjectLabels()
 {
     QHash<MapObject*, MapObjectLabel*> labelItems;
@@ -585,4 +699,120 @@ void ObjectSelectionItem::addRemoveObjectOutlines()
     mObjectOutlines.swap(outlineItems);
 }
 
+void ObjectSelectionItem::addRemoveObjectReferences()
+{
+    QHash<MapObject*, QList<ObjectReferenceItem*>> referencesBySourceObject;
+    QHash<MapObject*, QList<ObjectReferenceItem*>> referencesByTargetObject;
+    const MapRenderer &renderer = *mMapDocument->renderer();
+
+    auto ensureReferenceItem = [&] (MapObject *sourceObject, const QString &property, ObjectRef ref) {
+        MapObject *targetObject = DisplayObjectRef(ref, mMapDocument).object();
+        if (!targetObject)
+            return;
+
+        QList<ObjectReferenceItem*> &items = referencesBySourceObject[sourceObject];
+
+        if (mReferencesBySourceObject.contains(sourceObject)) {
+            QList<ObjectReferenceItem*> &existingItems = mReferencesBySourceObject[sourceObject];
+            auto it = std::find_if(existingItems.begin(),
+                                   existingItems.end(),
+                                   [=] (ObjectReferenceItem *item) {
+                return item->targetObject() == targetObject
+                        && item->property() == property;
+            });
+
+            if (it != existingItems.end()) {
+                items.append(*it);
+                referencesByTargetObject[targetObject].append(*it);
+
+                existingItems.erase(it);
+                return;
+            }
+        }
+
+        auto item = new ObjectReferenceItem(sourceObject, targetObject, property, this);
+        item->syncWithSourceObject(renderer);
+        item->syncWithTargetObject(renderer);
+        items.append(item);
+        referencesByTargetObject[targetObject].append(item);
+    };
+
+    if (Preferences::instance()->showObjectReferences()) {
+        LayerIterator iterator(mMapDocument->map());
+        while (Layer *layer = iterator.next()) {
+            if (layer->isHidden())
+                continue;
+
+            if (ObjectGroup *objectGroup = layer->asObjectGroup()) {
+                for (MapObject *object : objectGroup->objects()) {
+                    const auto &props = object->properties();
+                    for (auto it = props.cbegin(), it_end = props.cend(); it != it_end; ++it) {
+                        if (it->userType() == objectRefTypeId())
+                            ensureReferenceItem(object, it.key(), it->value<ObjectRef>());
+                    }
+                }
+            }
+        }
+    }
+
+    // delete remaining items
+    for (const auto &items : mReferencesBySourceObject)
+        qDeleteAll(items);
+
+    mReferencesBySourceObject.swap(referencesBySourceObject);
+    mReferencesByTargetObject.swap(referencesByTargetObject);
+}
+
+void ObjectSelectionItem::addRemoveObjectReferences(MapObject *object)
+{
+    QList<ObjectReferenceItem*> &items = mReferencesBySourceObject[object];
+    QList<ObjectReferenceItem*> existingItems;
+    items.swap(existingItems);
+
+    const MapRenderer &renderer = *mMapDocument->renderer();
+
+    auto ensureReferenceItem = [&] (MapObject *sourceObject, const QString &property, ObjectRef ref) {
+        MapObject *targetObject = DisplayObjectRef(ref, mMapDocument).object();
+        if (!targetObject)
+            return;
+
+        auto it = std::find_if(existingItems.begin(),
+                               existingItems.end(),
+                               [=] (ObjectReferenceItem *item) {
+            return item->targetObject() == targetObject
+                    && item->property() == property;
+        });
+
+        if (it != existingItems.end()) {
+            items.append(*it);
+            existingItems.erase(it);
+            return;
+        }
+
+        auto item = new ObjectReferenceItem(sourceObject, targetObject, property, this);
+        item->syncWithSourceObject(renderer);
+        item->syncWithTargetObject(renderer);
+        items.append(item);
+        mReferencesByTargetObject[targetObject].append(item);
+    };
+
+    if (Preferences::instance()->showObjectReferences()) {
+        const auto &props = object->properties();
+        for (auto it = props.cbegin(), it_end = props.cend(); it != it_end; ++it) {
+            if (it->userType() == objectRefTypeId())
+                ensureReferenceItem(object, it.key(), it->value<ObjectRef>());
+        }
+    }
+
+    // Delete remaining existing items, also removing them from mReferencesByTargetObject
+    for (ObjectReferenceItem *item : existingItems) {
+        auto &itemsByTarget = mReferencesByTargetObject[item->targetObject()];
+        itemsByTarget.removeOne(item);
+        if (itemsByTarget.isEmpty())
+            mReferencesByTargetObject.remove(item->targetObject());
+
+        delete item;
+    }
+}
+
 } // namespace Tiled
