@@ -32,19 +32,41 @@ namespace Tiled {
 class GroupLayer;
 class Layer;
 class MapObject;
+class MapRenderer;
 class Tile;
 class Tileset;
 
 class MapDocument;
 class MapObjectItem;
-class MapObjectLabel;
 class MapObjectOutline;
+class ObjectReferenceItem;
+
+class MapObjectLabel : public QGraphicsItem
+{
+public:
+    MapObjectLabel(const MapObject *object, QGraphicsItem *parent = nullptr);
+
+    const MapObject *mapObject() const { return mObject; }
+    void syncWithMapObject(const MapRenderer &renderer);
+    void updateColor();
+
+    QRectF boundingRect() const override;
+    void paint(QPainter *painter,
+               const QStyleOptionGraphicsItem *,
+               QWidget *) override;
+
+private:
+    QRectF mBoundingRect;
+    QPointF mTextPos;
+    const MapObject *mObject;
+    QColor mColor;
+};
 
 /**
  * A graphics item displaying object selection.
  *
- * Apart from selection outlines, it also displays name labels when
- * appropriate.
+ * Apart from selection outlines, it also displays name labels, hover highlight
+ * and object references.
  */
 class ObjectSelectionItem : public QGraphicsObject
 {
@@ -55,12 +77,18 @@ public:
                         QGraphicsItem *parent = nullptr);
     ~ObjectSelectionItem() override;
 
+    const MapRenderer &mapRenderer() const;
+
     // QGraphicsItem interface
     QRectF boundingRect() const override { return QRectF(); }
     void paint(QPainter *, const QStyleOptionGraphicsItem *, QWidget *) override {}
 
 private:
     void changeEvent(const ChangeEvent &event);
+    void propertyAdded(Object *object, const QString &name);
+    void propertyRemoved(Object *object, const QString &name);
+    void propertyChanged(Object *object, const QString &name);
+    void propertiesChanged(Object *object);
     void selectedObjectsChanged();
     void hoveredMapObjectChanged(MapObject *object, MapObject *previous);
     void mapChanged();
@@ -68,20 +96,26 @@ private:
     void layerAboutToBeRemoved(GroupLayer *parentLayer, int index);
     void layerChanged(Layer *layer);
     void syncOverlayItems(const QList<MapObject *> &objects);
-    void updateObjectLabelColors();
+    void updateItemColors() const;
     void objectsAdded(const QList<MapObject*> &objects);
     void objectsAboutToBeRemoved(const QList<MapObject*> &objects);
-    void tilesetTileOffsetChanged(Tileset *tileset);
+    void tilesetTilePositioningChanged(Tileset *tileset);
     void tileTypeChanged(Tile *tile);
 
     void objectLabelVisibilityChanged();
+    void showObjectReferencesChanged();
+    void objectLineWidthChanged();
 
     void addRemoveObjectLabels();
     void addRemoveObjectOutlines();
+    void addRemoveObjectReferences();
+    void addRemoveObjectReferences(MapObject *object);
 
     MapDocument *mMapDocument;
     QHash<MapObject*, MapObjectLabel*> mObjectLabels;
     QHash<MapObject*, MapObjectOutline*> mObjectOutlines;
+    QHash<MapObject*, QList<ObjectReferenceItem*>> mReferencesBySourceObject;
+    QHash<MapObject*, QList<ObjectReferenceItem*>> mReferencesByTargetObject;
     std::unique_ptr<MapObjectItem> mHoveredMapObjectItem;
 };
 
