@@ -26,13 +26,13 @@
 #include <QSet>
 #include <QVector>
 
+#include <memory>
+
 class QGraphicsItem;
 
 namespace Tiled {
-namespace Internal {
 
 class Handle;
-class MapObjectItem;
 class OriginIndicator;
 class ResizeHandle;
 class RotateHandle;
@@ -51,21 +51,26 @@ public:
 
     void keyPressed(QKeyEvent *) override;
     void mouseEntered() override;
+    void mouseLeft() override;
     void mouseMoved(const QPointF &pos,
                     Qt::KeyboardModifiers modifiers) override;
     void mousePressed(QGraphicsSceneMouseEvent *event) override;
     void mouseReleased(QGraphicsSceneMouseEvent *event) override;
+    void mouseDoubleClicked(QGraphicsSceneMouseEvent *event) override;
     void modifiersChanged(Qt::KeyboardModifiers modifiers) override;
 
     void languageChanged() override;
 
-private slots:
-    void updateHandles(bool resetOriginIndicator = true);
-    void updateHandleVisibility();
-
-    void objectsRemoved(const QList<MapObject *> &);
+protected:
+    void changeEvent(const ChangeEvent &event) override;
 
 private:
+    void updateHandles();
+    void updateHandlesAndOrigin();
+    void updateHandleVisibility();
+
+    void objectsAboutToBeRemoved(const QList<MapObject *> &);
+
     enum Action {
         NoAction,
         Selecting,
@@ -80,6 +85,8 @@ private:
         Rotate,
     };
 
+    void updateHandlesImpl(bool resetOriginIndicator);
+
     void updateHover(const QPointF &pos);
     void updateSelection(const QPointF &pos,
                          Qt::KeyboardModifiers modifiers);
@@ -111,6 +118,14 @@ private:
     void setMode(Mode mode);
     void saveSelectionState();
 
+    enum AbortReason {
+        UserInteraction,
+        Deactivated
+    };
+
+    void abortCurrentAction(AbortReason reason = UserInteraction,
+                            const QList<MapObject *> &removedObjects = QList<MapObject*>());
+
     void refreshCursor();
 
     QPointF snapToGrid(const QPointF &pos,
@@ -129,35 +144,33 @@ private:
         qreal oldRotation;
     };
 
-    SelectionRectangle *mSelectionRectangle;
-    QGraphicsItem *mOriginIndicator;
+    std::unique_ptr<SelectionRectangle> mSelectionRectangle;
+    std::unique_ptr<QGraphicsItem> mOriginIndicator;
     RotateHandle *mRotateHandles[4];
     ResizeHandle *mResizeHandles[8];
     bool mMousePressed;
 
-    MapObjectItem *mHoveredObjectItem;
+    MapObject *mHoveredObject;
     Handle *mHoveredHandle;
 
-    MapObjectItem *mClickedObjectItem;
+    MapObject *mClickedObject;
     OriginIndicator *mClickedOriginIndicator;
     RotateHandle *mClickedRotateHandle;
     ResizeHandle *mClickedResizeHandle;
 
     QVector<MovingObject> mMovingObjects;
 
-    QPointF mOldOriginPosition;
-
     QPointF mAlignPosition;
-    QPointF mOrigin;
+    QPointF mOriginPos;
     bool mResizingLimitHorizontal;
     bool mResizingLimitVertical;
     Mode mMode;
     Action mAction;
     QPointF mStart;
     QPointF mStartOffset;
+    QPointF mLastMousePos;
     QPoint mScreenStart;
     Qt::KeyboardModifiers mModifiers;
 };
 
-} // namespace Internal
 } // namespace Tiled
