@@ -26,12 +26,12 @@
 #include "documentmanager.h"
 #include "mapdocument.h"
 #include "mapobject.h"
-#include "preferences.h"
 #include "propertybrowser.h"
 #include "terrain.h"
 #include "tile.h"
 #include "tileset.h"
 #include "utils.h"
+#include "variantpropertymanager.h"
 
 #include <QAction>
 #include <QEvent>
@@ -41,6 +41,7 @@
 #include <QUndoStack>
 #include <QVBoxLayout>
 #include <QMenu>
+#include <QFileInfo>
 
 namespace Tiled {
 
@@ -340,7 +341,7 @@ void PropertiesDock::renamePropertyTo(const QString &name)
     undoStack->push(new RenameProperty(mDocument, mDocument->currentObjects(), oldName, name));
 }
 
-void PropertiesDock::showContextMenu(const QPoint& pos)
+void PropertiesDock::showContextMenu(const QPoint &pos)
 {
     const Object *object = mDocument->currentObject();
     if (!object)
@@ -360,13 +361,49 @@ void PropertiesDock::showContextMenu(const QPoint& pos)
     }
 
     QMenu contextMenu(mPropertyBrowser);
+
+    if (customPropertiesSelected && propertyNames.size() == 1) {
+        const auto value = object->resolvedProperty(propertyNames.first());
+        if (value.userType() == filePathTypeId()) {
+            const FilePath filePath = value.value<FilePath>();
+            const QString localFile = filePath.url.toLocalFile();
+
+            if (!localFile.isEmpty()) {
+                Utils::addOpenContainingFolderAction(contextMenu, localFile);
+
+                if (QFileInfo { localFile }.isFile())
+                    Utils::addOpenWithSystemEditorAction(contextMenu, filePath.url.toLocalFile());
+            }
+        } else if (value.userType() == objectRefTypeId()) {
+            if (auto mapDocument = qobject_cast<MapDocument*>(mDocument)) {
+                const DisplayObjectRef objectRef(value.value<ObjectRef>(), mapDocument);
+
+                contextMenu.addAction(tr("Go to Object"), [=] {
+                    if (auto object = objectRef.object()) {
+                        objectRef.mapDocument->setSelectedObjects({object});
+                        emit objectRef.mapDocument->focusMapObjectRequested(object);
+                    }
+                })->setEnabled(objectRef.object());
+            }
+        }
+    }
+
+    if (!contextMenu.isEmpty())
+        contextMenu.addSeparator();
+
     QAction *cutAction = contextMenu.addAction(tr("Cu&t"), this, &PropertiesDock::cutProperties);
     QAction *copyAction = contextMenu.addAction(tr("&Copy"), this, &PropertiesDock::copyProperties);
     QAction *pasteAction = contextMenu.addAction(tr("&Paste"), this, &PropertiesDock::pasteProperties);
     contextMenu.addSeparator();
-    QMenu *convertMenu = contextMenu.addMenu(tr("Convert To"));
-    QAction *renameAction = contextMenu.addAction(tr("Rename..."), this, &PropertiesDock::renameProperty);
-    QAction *removeAction = contextMenu.addAction(tr("Remove"), this, &PropertiesDock::removeProperties);
+    QMenu *convertMenu = nullptr;
+
+    if (customPropertiesSelected) {
+        convertMenu = contextMenu.addMenu(tr("Convert To"));
+        contextMenu.addAction(mActionRemoveProperty);
+        contextMenu.addAction(mActionRenameProperty);
+    } else {
+        contextMenu.addAction(mActionAddProperty);
+    }
 
     cutAction->setShortcuts(QKeySequence::Cut);
     cutAction->setIcon(QIcon(QLatin1String(":/images/16/edit-cut.png")));
@@ -377,24 +414,18 @@ void PropertiesDock::showContextMenu(const QPoint& pos)
     pasteAction->setShortcuts(QKeySequence::Paste);
     pasteAction->setIcon(QIcon(QLatin1String(":/images/16/edit-paste.png")));
     pasteAction->setEnabled(ClipboardManager::instance()->hasProperties());
-    renameAction->setEnabled(mActionRenameProperty->isEnabled());
-    renameAction->setIcon(mActionRenameProperty->icon());
-    removeAction->setEnabled(mActionRemoveProperty->isEnabled());
-    removeAction->setShortcuts(mActionRemoveProperty->shortcuts());
-    removeAction->setIcon(mActionRemoveProperty->icon());
 
     Utils::setThemeIcon(cutAction, "edit-cut");
     Utils::setThemeIcon(copyAction, "edit-copy");
     Utils::setThemeIcon(pasteAction, "edit-paste");
-    Utils::setThemeIcon(removeAction, "remove");
 
-
-    if (customPropertiesSelected) {
+    if (convertMenu) {
         const int convertTo[] = {
             QVariant::Bool,
             QVariant::Color,
             QVariant::Double,
             filePathTypeId(),
+            objectRefTypeId(),
             QVariant::Int,
             QVariant::String
         };
@@ -420,14 +451,15 @@ void PropertiesDock::showContextMenu(const QPoint& pos)
                 action->setData(toType);
             }
         }
+
+        convertMenu->setEnabled(!convertMenu->actions().isEmpty());
     }
 
-    convertMenu->setEnabled(!convertMenu->actions().isEmpty());
 
     const QPoint globalPos = mPropertyBrowser->mapToGlobal(pos);
     const QAction *selectedItem = contextMenu.exec(globalPos);
 
-    if (selectedItem && selectedItem->parentWidget() == convertMenu) {
+    if (selectedItem && convertMenu && selectedItem->parentWidget() == convertMenu) {
         QUndoStack *undoStack = mDocument->undoStack();
         undoStack->beginMacro(tr("Convert Property/Properties", nullptr, items.size()));
 
@@ -490,8 +522,12 @@ void PropertiesDock::retranslateUi()
     setWindowTitle(tr("Properties"));
 
     mActionAddProperty->setText(tr("Add Property"));
-    mActionRemoveProperty->setText(tr("Remove Property"));
-    mActionRenameProperty->setText(tr("Rename Property"));
+
+    mActionRemoveProperty->setText(tr("Remove"));
+    mActionRemoveProperty->setToolTip(tr("Remove Property"));
+
+    mActionRenameProperty->setText(tr("Rename..."));
+    mActionRenameProperty->setToolTip(tr("Rename Property"));
 }
 
 } // namespace Tiled
