@@ -130,6 +130,7 @@ private:
         FlippingProperty,
         DrawOrderProperty,
         FileNameProperty,
+        ObjectAlignmentProperty,
         TileOffsetProperty,
         MarginProperty,
         SpacingProperty,
@@ -144,7 +145,8 @@ private:
         TemplateProperty,
         CompressionLevelProperty,
         ChunkWidthProperty,
-        ChunkHeightProperty
+        ChunkHeightProperty,
+        TintColorProperty
     };
 
     void addMapProperties();
@@ -195,6 +197,9 @@ private:
     void updateCustomProperties();
     void updateCustomPropertyColor(const QString &name);
 
+    QVariant toDisplayValue(const QVariant &value) const;
+    QVariant fromDisplayValue(const QVariant &value) const;
+
     void retranslateUi();
 
     bool mUpdating;
@@ -221,6 +226,7 @@ private:
     QStringList mLayerFormatNames;
     QList<Map::LayerDataFormat> mLayerFormatValues;
     QStringList mRenderOrderNames;
+    QStringList mAlignmentNames;
     QStringList mFlippingFlagNames;
     QStringList mDrawOrderNames;
 };
