@@ -33,7 +33,6 @@
 #include <QCoreApplication>
 
 namespace Tiled {
-namespace Internal {
 
 void RaiseLowerHelper::raise()
 {
@@ -237,6 +236,8 @@ bool RaiseLowerHelper::initContext()
                                         Qt::AscendingOrder);
 
     for (QGraphicsItem *item : items) {
+        if (!item->isEnabled())
+            continue;
         if (MapObjectItem *mapObjectItem = qgraphicsitem_cast<MapObjectItem*>(item)) {
             if (mapObjectItem->mapObject()->objectGroup() == mObjectGroup)
                 mRelatedObjects.append(mapObjectItem->mapObject());
@@ -265,5 +266,4 @@ void RaiseLowerHelper::push(const QList<QUndoCommand*> &commands,
     undoStack->endMacro();
 }
 
-} // namespace Internal
 } // namespace Tiled
