@@ -20,12 +20,12 @@
 
 #include "replacetileset.h"
 
+#include "map.h"
 #include "mapdocument.h"
 
 #include <QCoreApplication>
 
 namespace Tiled {
-namespace Internal {
 
 ReplaceTileset::ReplaceTileset(MapDocument *mapDocument,
                                int index,
@@ -36,6 +36,7 @@ ReplaceTileset::ReplaceTileset(MapDocument *mapDocument,
     , mIndex(index)
     , mTileset(tileset)
 {
+    Q_ASSERT(mMapDocument->map()->tilesetAt(index) != tileset);
 }
 
 void ReplaceTileset::swap()
@@ -43,5 +44,4 @@ void ReplaceTileset::swap()
     mTileset = mMapDocument->replaceTileset(mIndex, mTileset);
 }
 
-} // namespace Internal
 } // namespace Tiled
