@@ -21,7 +21,6 @@
 #include "scriptmodule.h"
 
 #include "actionmanager.h"
-#include "commanddatamodel.h"
 #include "commandmanager.h"
 #include "editabletileset.h"
 #include "issuesmodel.h"
@@ -31,6 +30,7 @@
 #include "scriptedaction.h"
 #include "scriptedfileformat.h"
 #include "scriptedtool.h"
+#include "scriptfileformatwrappers.h"
 #include "scriptmanager.h"
 #include "tilesetdocument.h"
 #include "tileseteditor.h"
@@ -40,19 +40,23 @@
 #include <QInputDialog>
 #include <QMenu>
 #include <QMessageBox>
+#include <QQmlEngine>
 
 namespace Tiled {
 
 ScriptModule::ScriptModule(QObject *parent)
     : QObject(parent)
 {
-    auto documentManager = DocumentManager::instance();
-    connect(documentManager, &DocumentManager::documentCreated, this, &ScriptModule::documentCreated);
-    connect(documentManager, &DocumentManager::documentOpened, this, &ScriptModule::documentOpened);
-    connect(documentManager, &DocumentManager::documentAboutToBeSaved, this, &ScriptModule::documentAboutToBeSaved);
-    connect(documentManager, &DocumentManager::documentSaved, this, &ScriptModule::documentSaved);
-    connect(documentManager, &DocumentManager::documentAboutToClose, this, &ScriptModule::documentAboutToClose);
-    connect(documentManager, &DocumentManager::currentDocumentChanged, this, &ScriptModule::currentDocumentChanged);
+    // If the script module is only created for command-line use, there will
+    // not be a DocumentManager instance.
+    if (auto documentManager = DocumentManager::maybeInstance()) {
+        connect(documentManager, &DocumentManager::documentCreated, this, &ScriptModule::documentCreated);
+        connect(documentManager, &DocumentManager::documentOpened, this, &ScriptModule::documentOpened);
+        connect(documentManager, &DocumentManager::documentAboutToBeSaved, this, &ScriptModule::documentAboutToBeSaved);
+        connect(documentManager, &DocumentManager::documentSaved, this, &ScriptModule::documentSaved);
+        connect(documentManager, &DocumentManager::documentAboutToClose, this, &ScriptModule::documentAboutToClose);
+        connect(documentManager, &DocumentManager::currentDocumentChanged, this, &ScriptModule::currentDocumentChanged);
+    }
 }
 
 ScriptModule::~ScriptModule()
@@ -113,18 +117,50 @@ QStringList ScriptModule::menus() const
     return idsToNames(ActionManager::menus());
 }
 
+QStringList ScriptModule::mapFormats() const
+{
+    const auto formats = PluginManager::objects<MapFormat>();
+    QStringList ret;
+    ret.reserve(formats.length());
+    for (auto format : formats)
+        ret.append(format->shortName());
+
+    return ret;
+}
+
+QStringList ScriptModule::tilesetFormats() const
+{
+    const auto formats = PluginManager::objects<TilesetFormat>();
+    QStringList ret;
+    ret.reserve(formats.length());
+    for (auto format : formats)
+        ret.append(format->shortName());
+
+    return ret;
+}
+
 EditableAsset *ScriptModule::activeAsset() const
 {
-    auto documentManager = DocumentManager::instance();
-    if (Document *document = documentManager->currentDocument())
-        return document->editable();
+    if (auto documentManager = DocumentManager::maybeInstance())
+        if (Document *document = documentManager->currentDocument())
+            return document->editable();
 
     return nullptr;
 }
 
 bool ScriptModule::setActiveAsset(EditableAsset *asset) const
 {
-    auto documentManager = DocumentManager::instance();
+    if (!asset) {
+        ScriptManager::instance().throwNullArgError(0);
+        return false;
+    }
+
+    auto documentManager = DocumentManager::maybeInstance();
+    if (!documentManager) {
+        ScriptManager::instance().throwError(QCoreApplication::translate("Script Errors", "Editor not available"));
+        return false;
+    }
+
     for (const DocumentPtr &document : documentManager->documents())
         if (document->editable() == asset)
             return documentManager->switchToDocument(document.data());
@@ -134,26 +170,45 @@ bool ScriptModule::setActiveAsset(EditableAsset *asset) const
 
 QList<QObject *> ScriptModule::openAssets() const
 {
-    auto documentManager = DocumentManager::instance();
     QList<QObject *> assets;
-    for (const DocumentPtr &document : documentManager->documents())
-        assets.append(document->editable());
+    if (auto documentManager = DocumentManager::maybeInstance())
+        for (const DocumentPtr &document : documentManager->documents())
+            assets.append(document->editable());
     return assets;
 }
 
 TilesetEditor *ScriptModule::tilesetEditor() const
 {
-    return static_cast<TilesetEditor*>(DocumentManager::instance()->editor(Document::TilesetDocumentType));
+    if (auto documentManager = DocumentManager::maybeInstance())
+        return static_cast<TilesetEditor*>(documentManager->editor(Document::TilesetDocumentType));
+    return nullptr;
 }
 
 MapEditor *ScriptModule::mapEditor() const
 {
-    return static_cast<MapEditor*>(DocumentManager::instance()->editor(Document::MapDocumentType));
+    if (auto documentManager = DocumentManager::maybeInstance())
+        return static_cast<MapEditor*>(documentManager->editor(Document::MapDocumentType));
+    return nullptr;
+}
+
+FilePath ScriptModule::filePath(const QUrl &path) const
+{
+    return { path };
+}
+
+ObjectRef ScriptModule::objectRef(int id) const
+{
+    return { id };
 }
 
 EditableAsset *ScriptModule::open(const QString &fileName) const
 {
-    auto documentManager = DocumentManager::instance();
+    auto documentManager = DocumentManager::maybeInstance();
+    if (!documentManager) {
+        ScriptManager::instance().throwError(QCoreApplication::translate("Script Errors", "Editor not available"));
+        return nullptr;
+    }
+
     documentManager->openFile(fileName);
 
     // If opening succeeded, it is the current document
@@ -167,9 +222,17 @@ EditableAsset *ScriptModule::open(const QString &fileName) const
 
 bool ScriptModule::close(EditableAsset *asset) const
 {
-    auto documentManager = DocumentManager::instance();
+    if (!asset) {
+        ScriptManager::instance().throwNullArgError(0);
+        return false;
+    }
+
+    auto documentManager = DocumentManager::maybeInstance();
+    int index = -1;
+
+    if (documentManager)
+        index = documentManager->findDocument(asset->document());
 
-    int index = documentManager->findDocument(asset->document());
     if (index == -1) {
         ScriptManager::instance().throwError(QCoreApplication::translate("Script Errors", "Not an open asset"));
         return false;
@@ -181,9 +244,17 @@ bool ScriptModule::close(EditableAsset *asset) const
 
 EditableAsset *ScriptModule::reload(EditableAsset *asset) const
 {
-    auto documentManager = DocumentManager::instance();
+    if (!asset) {
+        ScriptManager::instance().throwNullArgError(0);
+        return nullptr;
+    }
+
+    auto documentManager = DocumentManager::maybeInstance();
+    int index = -1;
+
+    if (documentManager)
+        index = documentManager->findDocument(asset->document());
 
-    int index = documentManager->findDocument(asset->document());
     if (index == -1) {
         ScriptManager::instance().throwError(QCoreApplication::translate("Script Errors", "Not an open asset"));
         return nullptr;
@@ -282,6 +353,51 @@ QJSValue ScriptModule::registerTool(const QString &shortName, QJSValue toolObjec
     return toolObject;
 }
 
+ScriptMapFormatWrapper *ScriptModule::mapFormat(const QString &shortName) const
+{
+    const auto formats = PluginManager::objects<MapFormat>();
+    for (auto format : formats) {
+        if (format->shortName() == shortName)
+            return new ScriptMapFormatWrapper(format);
+    }
+
+    return nullptr;
+}
+
+ScriptMapFormatWrapper *ScriptModule::mapFormatForFile(const QString &fileName) const
+{
+    const auto formats = PluginManager::objects<MapFormat>();
+    for (auto format : formats) {
+        if (format->supportsFile(fileName))
+            return new ScriptMapFormatWrapper(format);
+    }
+
+    return nullptr;
+}
+
+ScriptTilesetFormatWrapper *ScriptModule::tilesetFormat(const QString &shortName) const
+{
+    const auto formats = PluginManager::objects<TilesetFormat>();
+    for (auto format : formats) {
+        if (format->shortName() == shortName)
+            return new ScriptTilesetFormatWrapper(format);
+    }
+
+    return nullptr;
+}
+
+ScriptTilesetFormatWrapper *ScriptModule::tilesetFormatForFile(const QString &fileName) const
+{
+    const auto formats = PluginManager::objects<TilesetFormat>();
+    for (auto format : formats) {
+        if (format->supportsFile(fileName))
+            return new ScriptTilesetFormatWrapper(format);
+    }
+
+    return nullptr;
+}
+
+
 static QString toString(QJSValue value)
 {
     if (value.isString())
@@ -370,9 +486,9 @@ void ScriptModule::trigger(const QByteArray &actionName) const
 
 void ScriptModule::executeCommand(const QString &name, bool inTerminal) const
 {
-    auto commandDataModel = CommandManager::instance()->commandDataModel();
+    const auto commands = CommandManager::instance()->allCommands();
 
-    for (const Command &command : commandDataModel->allCommands()) {
+    for (const Command &command : commands) {
         if (command.name == name) {
             command.execute(inTerminal);
             return;
