@@ -43,10 +43,20 @@ void SelectSameTileTool::tilePositionChanged(QPoint tilePos)
     if (!tileLayer)
         return;
 
+    const bool infinite = mapDocument()->map()->infinite();
+
     QRegion resultRegion;
-    if (mapDocument()->map()->infinite() || tileLayer->contains(tilePos)) {
+    if (infinite || tileLayer->contains(tilePos)) {
         const Cell &matchCell = tileLayer->cellAt(tilePos);
-        resultRegion = tileLayer->region([&] (const Cell &cell) { return cell == matchCell; });
+        if (matchCell.isEmpty()) {
+            // Due to the way TileLayer::region only iterates allocated chunks,
+            // and because of different desired behavior for infinite vs. fixed
+            // maps we need a special handling when matching the empty cell.
+            resultRegion = infinite ? tileLayer->bounds() : tileLayer->rect();
+            resultRegion -= tileLayer->region();
+        } else {
+            resultRegion = tileLayer->region([&] (const Cell &cell) { return cell == matchCell; });
+        }
     }
     setSelectedRegion(resultRegion);
     brushItem()->setTileRegion(selectedRegion());
