@@ -29,10 +29,8 @@
 #include <QHBoxLayout>
 #include <QLabel>
 #include <QToolButton>
-#include <QFileInfo>
 
 namespace Tiled {
-namespace Internal {
 
 TilesetParametersEdit::TilesetParametersEdit(QWidget *parent)
     : QWidget(parent)
@@ -62,7 +60,7 @@ void TilesetParametersEdit::setTilesetDocument(TilesetDocument *tilesetDocument)
     mTilesetDocument = tilesetDocument;
 
     if (tilesetDocument)
-        mLabel->setText(QFileInfo(tilesetDocument->tileset()->imageSource()).fileName());
+        mLabel->setText(tilesetDocument->tileset()->imageSource().fileName());
     else
         mLabel->clear();
 }
@@ -85,5 +83,4 @@ void TilesetParametersEdit::buttonClicked()
     }
 }
 
-} // namespace Internal
 } // namespace Tiled
