@@ -20,9 +20,14 @@
 
 #pragma once
 
+#include "rangeset.h"
+
 #include <QIcon>
+#include <QSettings>
 #include <QString>
 
+#include <memory>
+
 class QAction;
 class QKeyEvent;
 class QMenu;
@@ -36,6 +41,10 @@ QString writableImageFormatsFilter();
 QStringList cleanFilterList(const QString &filter);
 bool fileNameMatchesNameFilter(const QString &fileName,
                                const QString &nameFilter);
+QString firstExtension(const QString &nameFilter);
+
+int matchingScore(const QStringList &words, QStringRef string);
+RangeSet<int> matchingRanges(const QStringList &words, QStringRef string);
 
 /**
  * Looks up the icon with the specified \a name from the system theme and set
@@ -68,6 +77,7 @@ void setThemeIcon(T *t, const char *name)
 void restoreGeometry(QWidget *widget);
 void saveGeometry(QWidget *widget);
 
+int defaultDpi();
 qreal defaultDpiScale();
 int dpiScaled(int value);
 qreal dpiScaled(qreal value);
@@ -81,6 +91,11 @@ bool isZoomOutShortcut(QKeyEvent *event);
 bool isResetZoomShortcut(QKeyEvent *event);
 
 void addFileManagerActions(QMenu &menu, const QString &fileName);
+void addOpenContainingFolderAction(QMenu &menu, const QString &fileName);
+void addOpenWithSystemEditorAction(QMenu &menu, const QString &fileName);
+
+QSettings::Format jsonSettingsFormat();
+std::unique_ptr<QSettings> jsonSettings(const QString &fileName);
 
 } // namespace Utils
 } // namespace Tiled
