@@ -21,7 +21,11 @@
 
 #include "variantpropertymanager.h"
 
+#include "documentmanager.h"
 #include "mapdocument.h"
+#include "mapobject.h"
+#include "mapobjectmodel.h"
+#include "objectgroup.h"
 #include "textpropertyedit.h"
 #include "tilesetdocument.h"
 
@@ -30,19 +34,27 @@
 namespace Tiled {
 
 class TilesetParametersPropertyType {};
-class AlignmentPropertyType {};
 
 } // namespace Tiled
 
 // Needs to be up here rather than at the bottom of the file to make a
 // static_assert in qMetaTypeId work (as of C++11)
 Q_DECLARE_METATYPE(Tiled::TilesetParametersPropertyType)
-Q_DECLARE_METATYPE(Tiled::AlignmentPropertyType)
 
 namespace Tiled {
 
+MapObject *DisplayObjectRef::object() const
+{
+    if (!mapDocument || ref.id <= 0)
+        return nullptr;
+    return mapDocument->map()->findObjectById(ref.id);
+}
+
+
 VariantPropertyManager::VariantPropertyManager(QObject *parent)
     : QtVariantPropertyManager(parent)
+    , mFilterAttribute(QStringLiteral("filter"))
+    , mDirectoryAttribute(QStringLiteral("directory"))
     , mSuggestionsAttribute(QStringLiteral("suggestions"))
     , mMultilineAttribute(QStringLiteral("multiline"))
     , mImageMissingIcon(QStringLiteral("://images/16/image-missing.png"))
@@ -58,7 +70,7 @@ VariantPropertyManager::VariantPropertyManager(QObject *parent)
 QVariant VariantPropertyManager::value(const QtProperty *property) const
 {
     if (mValues.contains(property))
-        return mValues[property].value;
+        return mValues[property];
     if (m_alignValues.contains(property))
         return QVariant::fromValue(m_alignValues.value(property));
     return QtVariantPropertyManager::value(property);
@@ -66,11 +78,10 @@ QVariant VariantPropertyManager::value(const QtProperty *property) const
 
 bool VariantPropertyManager::isPropertyTypeSupported(int propertyType) const
 {
-    if (propertyType == filePathTypeId())
-        return true;
-    if (propertyType == tilesetParametersTypeId())
-        return true;
-    if (propertyType == alignmentTypeId())
+    if (propertyType == filePathTypeId()
+            || propertyType == displayObjectRefTypeId()
+            || propertyType == tilesetParametersTypeId()
+            || propertyType == alignmentTypeId())
         return true;
     return QtVariantPropertyManager::isPropertyTypeSupported(propertyType);
 }
@@ -78,19 +89,22 @@ bool VariantPropertyManager::isPropertyTypeSupported(int propertyType) const
 int VariantPropertyManager::valueType(int propertyType) const
 {
     if (propertyType == filePathTypeId())
-        return QVariant::String;
+        return propertyType;
+    if (propertyType == displayObjectRefTypeId())
+        return propertyType;
     if (propertyType == tilesetParametersTypeId())
         return qMetaTypeId<TilesetDocument*>();
     if (propertyType == alignmentTypeId())
-        return qMetaTypeId<Qt::Alignment>();
+        return propertyType;
     return QtVariantPropertyManager::valueType(propertyType);
 }
 
 QStringList VariantPropertyManager::attributes(int propertyType) const
 {
     if (propertyType == filePathTypeId()) {
-        return QStringList {
-            QStringLiteral("filter")
+        return {
+            mFilterAttribute,
+            mDirectoryAttribute
         };
     }
     return QtVariantPropertyManager::attributes(propertyType);
@@ -100,8 +114,10 @@ int VariantPropertyManager::attributeType(int propertyType,
                                           const QString &attribute) const
 {
     if (propertyType == filePathTypeId()) {
-        if (attribute == QLatin1String("filter"))
+        if (attribute == mFilterAttribute)
             return QVariant::String;
+        if (attribute == mDirectoryAttribute)
+            return QVariant::Bool;
         return 0;
     }
     return QtVariantPropertyManager::attributeType(propertyType, attribute);
@@ -110,9 +126,11 @@ int VariantPropertyManager::attributeType(int propertyType,
 QVariant VariantPropertyManager::attributeValue(const QtProperty *property,
                                                 const QString &attribute) const
 {
-    if (mValues.contains(property)) {
-        if (attribute == QLatin1String("filter"))
-            return mValues[property].filter;
+    if (mFilePathAttributes.contains(property)) {
+        if (attribute == mFilterAttribute)
+            return mFilePathAttributes[property].filter;
+        if (attribute == mDirectoryAttribute)
+            return mFilePathAttributes[property].directory;
         return QVariant();
     }
     if (mStringAttributes.contains(property)) {
@@ -132,15 +150,47 @@ int VariantPropertyManager::tilesetParametersTypeId()
 
 int VariantPropertyManager::alignmentTypeId()
 {
-    return qMetaTypeId<AlignmentPropertyType>();
+    return qMetaTypeId<Qt::Alignment>();
+}
+
+int VariantPropertyManager::displayObjectRefTypeId()
+{
+    return qMetaTypeId<DisplayObjectRef>();
+}
+
+QString VariantPropertyManager::objectRefLabel(const MapObject *object) const
+{
+    QString label = tr("%1: ").arg(object->id());
+    if (!object->name().isEmpty()) {
+        label.append(object->name());
+        if (!object->type().isEmpty())
+            label.append(tr(" (%1)").arg(object->type()));
+    } else if (!object->type().isEmpty())
+        label.append(tr("(%1)").arg(object->type()));
+    else
+        label.append(tr("Unnamed object"));
+
+    return label;
 }
 
 QString VariantPropertyManager::valueText(const QtProperty *property) const
 {
     if (mValues.contains(property)) {
-        QVariant value = mValues[property].value;
+        QVariant value = mValues[property];
         int typeId = propertyType(property);
 
+        if (typeId == displayObjectRefTypeId()) {
+            const auto ref = value.value<DisplayObjectRef>();
+
+            if (ref.id() == 0)
+                return tr("Unset");
+
+            if (auto object = ref.object())
+                return objectRefLabel(object);
+
+            return tr("%1: Object not found").arg(QString::number(ref.id()));
+        }
+
         if (typeId == filePathTypeId()) {
             FilePath filePath = value.value<FilePath>();
             QString fileName = filePath.url.fileName();
@@ -179,7 +229,7 @@ QString VariantPropertyManager::valueText(const QtProperty *property) const
 QIcon VariantPropertyManager::valueIcon(const QtProperty *property) const
 {
     if (mValues.contains(property)) {
-        QVariant value = mValues[property].value;
+        QVariant value = mValues[property];
         QString filePath;
         int typeId = propertyType(property);
 
@@ -194,6 +244,12 @@ QIcon VariantPropertyManager::valueIcon(const QtProperty *property) const
                 filePath = tilesetDocument->tileset()->imageSource().toLocalFile();
         }
 
+        if (typeId == displayObjectRefTypeId()) {
+            const DisplayObjectRef ref = value.value<DisplayObjectRef>();
+            if (auto object = ref.object())
+                return ObjectIconManager::instance().iconForObject(object);
+        }
+
         // TODO: This assumes the file path is an image reference. It should be
         // replaced with a more generic icon.
         if (filePath.isEmpty() || !QFile::exists(filePath))
@@ -208,11 +264,10 @@ QIcon VariantPropertyManager::valueIcon(const QtProperty *property) const
 void VariantPropertyManager::setValue(QtProperty *property, const QVariant &value)
 {
     if (mValues.contains(property)) {
-        Data d = mValues[property];
-        if (d.value == value)
+        QVariant &storedValue = mValues[property];
+        if (storedValue == value)
             return;
-        d.value = value;
-        mValues[property] = d;
+        storedValue = value;
         emit propertyChanged(property);
         emit valueChanged(property, value);
         return;
@@ -248,17 +303,22 @@ void VariantPropertyManager::setAttribute(QtProperty *property,
                                           const QString &attribute,
                                           const QVariant &val)
 {
-    if (mValues.contains(property)) {
-        if (attribute == QLatin1String("filter")) {
+    if (mFilePathAttributes.contains(property)) {
+        FilePathAttributes &attributes = mFilePathAttributes[property];
+        if (attribute == mFilterAttribute) {
             if (val.type() != QVariant::String && !val.canConvert(QVariant::String))
                 return;
-            QString str = val.toString();
-            Data d = mValues[property];
-            if (d.filter == str)
+            QString filter = val.toString();
+            if (attributes.filter == filter)
                 return;
-            d.filter = str;
-            mValues[property] = d;
-            emit attributeChanged(property, attribute, str);
+            attributes.filter = filter;
+            emit attributeChanged(property, attribute, filter);
+        } else if (attribute == mDirectoryAttribute) {
+            bool directory = val.toBool();
+            if (attributes.directory == directory)
+                return;
+            attributes.directory = directory;
+            emit attributeChanged(property, attribute, directory);
         }
         return;
     }
@@ -280,10 +340,12 @@ void VariantPropertyManager::setAttribute(QtProperty *property,
 void VariantPropertyManager::initializeProperty(QtProperty *property)
 {
     const int type = propertyType(property);
-    if (type == filePathTypeId()) {
-        mValues[property] = Data();
-    } else if (type == tilesetParametersTypeId()) {
-        mValues[property] = Data();
+    if (type == filePathTypeId()
+            || type == displayObjectRefTypeId()
+            || type == tilesetParametersTypeId()) {
+        mValues[property] = QVariant();
+        if (type == filePathTypeId())
+            mFilePathAttributes[property] = FilePathAttributes();
     } else if (type == QVariant::String) {
         mStringAttributes[property] = StringAttributes();
     } else if (type == alignmentTypeId()) {
@@ -316,6 +378,7 @@ void VariantPropertyManager::initializeProperty(QtProperty *property)
 void VariantPropertyManager::uninitializeProperty(QtProperty *property)
 {
     mValues.remove(property);
+    mFilePathAttributes.remove(property);
     mStringAttributes.remove(property);
     m_alignValues.remove(property);
 
