@@ -21,12 +21,35 @@
 
 #pragma once
 
+#include "properties.h"
+
 #include <QtVariantPropertyManager>
 
 #include <QFileIconProvider>
 
 namespace Tiled {
 
+class MapDocument;
+class MapObject;
+
+class DisplayObjectRef {
+public:
+    explicit DisplayObjectRef(ObjectRef ref = ObjectRef(),
+                              MapDocument *mapDocument = nullptr)
+        : ref(ref)
+        , mapDocument(mapDocument)
+    {}
+
+    bool operator==(const DisplayObjectRef &o) const
+    { return ref.id == o.ref.id && mapDocument == o.mapDocument; }
+
+    int id() const { return ref.id; }
+    MapObject *object() const;
+
+    ObjectRef ref;
+    MapDocument *mapDocument;
+};
+
 /**
  * Extension of the QtVariantPropertyManager that adds support for a filePath
  * data type.
@@ -49,6 +72,7 @@ public:
 
     static int tilesetParametersTypeId();
     static int alignmentTypeId();
+    static int displayObjectRefTypeId();
 
 public slots:
     void setValue(QtProperty *property, const QVariant &val) override;
@@ -65,12 +89,15 @@ protected:
 private:
     void slotValueChanged(QtProperty *property, const QVariant &value);
     void slotPropertyDestroyed(QtProperty *property);
+    QString objectRefLabel(const MapObject *object) const;
 
-    struct Data {
-        QVariant value;
+    QMap<const QtProperty *, QVariant> mValues;
+
+    struct FilePathAttributes {
         QString filter;
+        bool directory = false;
     };
-    QMap<const QtProperty *, Data> mValues;
+    QMap<const QtProperty *, FilePathAttributes> mFilePathAttributes;
 
     struct StringAttributes {
         QStringList suggestions;
@@ -91,6 +118,8 @@ private:
     PropertyToPropertyMap m_alignHToProperty;
     PropertyToPropertyMap m_alignVToProperty;
 
+    const QString mFilterAttribute;
+    const QString mDirectoryAttribute;
     const QString mSuggestionsAttribute;
     const QString mMultilineAttribute;
     QIcon mImageMissingIcon;
@@ -98,3 +127,5 @@ private:
 };
 
 } // namespace Tiled
+
+Q_DECLARE_METATYPE(Tiled::DisplayObjectRef)
