@@ -28,8 +28,6 @@ namespace Tiled {
 
 class Tileset;
 
-namespace Internal {
-
 class TilesetDocument;
 
 class WangColorModel : public QAbstractItemModel
@@ -81,13 +79,15 @@ public:
     void setProbability(WangColor *wangColor, qreal probability);
 
 private:
+    enum IndexTypeId {
+        CornerIndexId = 1,
+        EdgeIndexId = 2
+    };
+
     void emitDataChanged(WangColor *wangColor);
 
     TilesetDocument *mTilesetDocument;
     WangSet *mWangSet;
-    QString *mEdgeText;
-    QString *mCornerText;
 };
 
-} // namespace Internal
 } // namespace Tiled
