@@ -2,9 +2,9 @@
  * #%L
  * This file is part of libtiled-java.
  * %%
- * Copyright (C) 2004 - 2017 Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
- * Copyright (C) 2004 - 2017 Adam Turk <aturk@biggeruniverse.com>
- * Copyright (C) 2016 - 2017 Mike Thomas <mikepthomas@outlook.com>
+ * Copyright (C) 2004 - 2019 Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
+ * Copyright (C) 2004 - 2019 Adam Turk <aturk@biggeruniverse.com>
+ * Copyright (C) 2016 - 2019 Mike Thomas <mikepthomas@outlook.com>
  * %%
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -67,10 +67,7 @@ import org.mapeditor.io.xml.XMLWriter;
 /**
  * A writer for Tiled's TMX map format.
  *
- * @author Thorbjørn Lindeijer
- * @author Adam Turk
- * @author Mike Thomas
- * @version 1.0.2
+ * @version 1.2.3
  */
 public class TMXMapWriter {
 
@@ -139,7 +136,7 @@ public class TMXMapWriter {
     }
 
     /**
-     * <p>writeMap.</p>
+     * writeMap.
      *
      * @param map a {@link org.mapeditor.core.Map} object.
      * @param out a {@link java.io.OutputStream} object.
@@ -157,7 +154,7 @@ public class TMXMapWriter {
     }
 
     /**
-     * <p>writeTileset.</p>
+     * writeTileset.
      *
      * @param set a {@link org.mapeditor.core.TileSet} object.
      * @param out a {@link java.io.OutputStream} object.
@@ -309,10 +306,17 @@ public class TMXMapWriter {
             // Write tile properties when necessary.
             for (Tile tile : set) {
                 // todo: move the null check back into the iterator?
-                if (tile != null && !tile.getProperties().isEmpty()) {
+                if (tile != null
+                        && (!tile.getProperties().isEmpty()
+                        || !tile.getType().isEmpty())) {
                     w.startElement("tile");
                     w.writeAttribute("id", tile.getId());
-                    writeProperties(tile.getProperties(), w);
+                    if (!tile.getType().isEmpty()) {
+                        w.writeAttribute("type", tile.getType());
+                    }
+                    if (!tile.getProperties().isEmpty()) {
+                        writeProperties(tile.getProperties(), w);
+                    }
                     w.endElement();
                 }
             }
@@ -324,6 +328,7 @@ public class TMXMapWriter {
             // TODO: This shouldn't be necessary
             for (Tile tile : set) {
                 if (!tile.getProperties().isEmpty()
+                        || !tile.getType().isEmpty()
                         || tile.getSource() != null) {
                     needWrite = true;
                     break;
@@ -527,6 +532,10 @@ public class TMXMapWriter {
         w.startElement("tile");
         w.writeAttribute("id", tile.getId());
 
+        if (!tile.getType().isEmpty()) {
+            w.writeAttribute("type", tile.getType());
+        }
+
         if (!tile.getProperties().isEmpty()) {
             writeProperties(tile.getProperties(), w);
         }
@@ -682,7 +691,7 @@ public class TMXMapWriter {
     }
 
     /**
-     * <p>accept.</p>
+     * accept.
      *
      * @param pathName a {@link java.io.File} object.
      * @return a boolean.
